/** 
 * Ce fichier contient diverses actions qui peuvent s'appliquer sur un Document
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 * module Actions.java 
 *
 * Programme kartoMNT : cartographie 2D sur cartes numriques
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 * Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 * y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 * ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.text.html.*;

import javax.swing.AbstractAction;
import javax.swing.*;
import com.sun.j3d.utils.applet.JMainFrame;
import java.util.*;
import java.io.IOException;
import java.awt.Dimension;

import kartoMNT.visu3d.*;

/**
 * Zoom sur les documents
 */
class ActionZoom extends AbstractAction2 implements SelectDocumentListener
	{

		protected double zoom;

		Document currentDocument;
		protected DocumentKartoContainer documentKartoContainer;
		protected DocumentMNTContainer documentMNTContainer;
		
		ActionZoom(DocumentKartoContainer c1,DocumentMNTContainer c2,double _zoom)
		{
			super(ResourceManager.get("zoom")+" "+(int)(_zoom*100)+"%");
			zoom=_zoom;

			documentKartoContainer=c1;
			documentMNTContainer=c2;
			documentKartoContainer.addSelectDocumentKartoListener(this);
			documentMNTContainer.addSelectDocumentMNTListener(this);

		}

		public void documentSelected(Document doc)
		{
			currentDocument=doc;
			if(doc==null)
				{
					setEnabled(false);
					return;
				}
			setEnabled(true);

			if((zoom==doc.getScaleX())
					&&(zoom==doc.getScaleY())
			  )
				{
					setSelected(true);

				}
			else
				{
					setSelected(false);
				}
		}


		public void actionPerformed(ActionEvent e)
		{
			if(currentDocument!=null)
				{
					if(getSelected())
						{
							currentDocument.setScale(zoom,zoom);
						}
				}


		}

	   
	}


/**
 * Selection du type d'interpolation pendant le zoom
 * (Plus proche voisin ou bilineaire)
 */
class ActionSetInterpolationType extends AbstractAction2 implements SelectDocumentListener
	{
		Document currentDocument;
		protected DocumentKartoContainer documentKartoContainer;
		protected DocumentMNTContainer documentMNTContainer;
		
		public ActionSetInterpolationType(DocumentKartoContainer c1,DocumentMNTContainer c2)
		{
			super(ResourceManager.get("zoomInterpolation"));
			documentKartoContainer=c1;
			documentMNTContainer=c2;
			documentKartoContainer.addSelectDocumentKartoListener(this);
			documentMNTContainer.addSelectDocumentMNTListener(this);

		}

		public void documentSelected(Document doc)
		{
			currentDocument=doc;
			if(doc==null)
				{
					setEnabled(false);
					return;
				}
			setEnabled(true);

			if(doc.getInterpolationType()==ImageCanvas.TYPE_BILINEAR)
				{
					setSelected(true);
				}
			else
				{
					setSelected(false);
				}
		}


		public void actionPerformed(ActionEvent e)
		{
			
			try
				{
					if(getSelected())
						currentDocument.setInterpolationType(ImageCanvas.TYPE_BILINEAR);
					else
						currentDocument.setInterpolationType(ImageCanvas.TYPE_NEAREST_NEIGHBOR);
				}
			catch(NullPointerException z)
			{}
		}
	   
	}



/**
 * Ouverture d'une fentre de fusion/modification de MNT (FusionMNTDialog)
 */
class ActionFusion extends AbstractAction
	{
		//protected String dir;
		protected DocumentKartoContainer KmainWindow;
		protected DocumentMNTContainer MmainWindow;
		protected JFrame parent;
		//protected ActionImportImage importIm;
		
		JDialog fusionDialog;
		
		public ActionFusion(JFrame p,DocumentKartoContainer kw, DocumentMNTContainer mw)
		{
			super(ResourceManager.get("MNTCreate"));
			setEnabled(true);
		  
			KmainWindow=kw;
			MmainWindow=mw;
			parent=p;
		}

		public void documentSelected(Document doc)
		{}

		public void actionPerformed(ActionEvent e)
		{
			fusionDialog = new FusionMNTDialog(parent,KmainWindow,MmainWindow);
			fusionDialog.show();
			
		}

	}



/**
 * Lancement du visualisateur 3D de MNT
 */
class ActionVisu3D extends AbstractAction2
	{
		//protected String dir;
		protected DocumentKartoContainer KmainWindow;
		protected DocumentMNTContainer MmainWindow;
		protected JFrame parent;
		//protected ActionImportImage importIm;				
		
		public ActionVisu3D(JFrame p,DocumentKartoContainer kw, DocumentMNTContainer mw)
		{
			super(ResourceManager.get("visu3D"));
			setEnabled(true);
		  
			KmainWindow=kw;
			MmainWindow=mw;
			parent=p;
		}

		public void documentSelected(Document doc)
		{}

		public void actionPerformed(ActionEvent e)
		{
			System.out.println("\nVision 3d");
			Visu3DKartoMNT d;
			MNT land = null;
			if (MmainWindow.getCurrentDocumentMNT()!=null)
			land = MmainWindow.getCurrentDocumentMNT().getMNT();
			JFrame frame = new JMainFrame2(d=new Visu3DKartoMNT(land,KmainWindow.getCurrentDocumentKarto()), 800, 600);
			if (KmainWindow.getCurrentDocumentKarto()!=null)
				frame.setTitle("KartoMNT V2 ["+KmainWindow.getCurrentDocumentKarto().getName()+"]");
			else
				frame.setTitle("KartoMNT V2 ");
		}

	}


/**
 * Ouverture d'une fentre d'aide
 * Cette classe n'est entirement pas implmente. 
 * Elle renvoie provisoirement  la page internet de karto : http://karto.free.fr
 */
class ActionHelp extends AbstractAction {
	
	public ActionHelp(){
		super(ResourceManager.get("kartoHelp"));
						
	}
	
		public void actionPerformed(ActionEvent e){
		
   		HelpWindow hw = new HelpWindow(ResourceManager.get("help"));
	   		//hw.show();
	   		hw.setVisible(true);
	}  
	
	
	class HelpWindow extends JFrame implements HyperlinkListener {
 		 
 		 
 		private JEditorPane helpPane;
		
 		private java.io.File f = new java.io.File("doc/DocUtilisateur.html");
		private java.net.URL url;

 		public HelpWindow(String s){
 			 
 			super(s);
			try{
 				url = f.toURL();
				helpPane = new JEditorPane(url);
  				helpPane.setEditable(false);
				helpPane.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
  				getContentPane().add(new JScrollPane(helpPane));
 				helpPane.setPreferredSize(new Dimension(800,500));
	   				helpPane.addHyperlinkListener(this);
				pack();
	   		} catch(IOException ioe) {
					// Some warning to user
	  		} 
	   			
		}
		
		public void hyperlinkUpdate(HyperlinkEvent event) {
			if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
	 			try {
					helpPane.setPage(event.getURL());
	  			} catch(IOException ioe) {
					
	  			}
			}
  		}

		
		
	 	};

	
}
