/** 
 * Ce fichier contient l'ensemble des actions qui peuvent s'appliquer sur un DocumentKarto
 * En particulier, l'exportation
 *
 * @author HEIT Sylvestre & Coustillac Laurent
 * module ActionsMNT.java
 *
 * Programme kartoMNT : cartographie 2D sur cartes numriques
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.awt.geom.*;


/**
 * Sauvegarde du MNT
 * Cette classe utilise GestionnaireMntFormat afin de proposer tous les formats de MNT dtects
 */
class ActionExportMNT extends DocumentMNTAction
    {
        protected DocumentKartoContainer mainWindow;
		JFrame parent;
        protected String dir;
        
        public ActionExportMNT(JFrame _parent,DocumentMNTContainer w)
        {
            super(w,ResourceManager.get("saveMNT"));
            parent=_parent;

            dir="";

            File f = new File (".");
            try
                {
                    dir=f.getCanonicalPath();
                }
            catch(Exception ex)
                {
                    System.out.println(ex.getMessage());
                }
        }

        /**
         * modifier cette mthode pour ajouter de nouvelles classes pour les exportations
         */
         ///***///
        public void actionPerformed(ActionEvent e)
        {
            if(getCurrentDocument()!=null)
                {
                    {
                        MNT mnt=(getCurrentDocument()).getMNT();

                        if(mnt!=null)
                            {
                                try
                                    {

                                        JFileChooser chooser = new JFileChooser(dir);
                                        chooser.setAcceptAllFileFilterUsed(false);

                                        /*ExampleFileFilter filter1 = new ExampleFileFilter();
                                        filter1.addExtension("sur");
                                        filter1.setDescription(ResourceManager.get("ModelNT"));

                                        ExampleFileFilter filter2 = new ExampleFileFilter();
                                        filter2.addExtension("wrl");
                                        filter2.setDescription(ResourceManager.get("vrmlRep"));

                                        ExampleFileFilter filter3 = new ExampleFileFilter();
                                        filter3.addExtension("mnt");
                                        filter3.setDescription(ResourceManager.get("MNTbin"));

                                        chooser.addChoosableFileFilter(filter2);
                                        chooser.addChoosableFileFilter(filter1);
                                        chooser.addChoosableFileFilter(filter3);*/

					GestionnaireMntFormat.initFilter(chooser,MntFormat.EXPORT);

                                        int returnVal = chooser.showSaveDialog(parent);

                                        if(returnVal == JFileChooser.APPROVE_OPTION)
                                            {
                                                /*ExportMNT exportMNT;
                                                String fileName="";
                                                if(chooser.getFileFilter()==filter1)
                                                    {
                                                        exportMNT=new ExportSUR();
                                                        fileName=chooser.getSelectedFile().getPath()+".sur";
                                                        exportMNT.export(mnt, fileName);
                                                    }
                                                if(chooser.getFileFilter()==filter2)
                                                    {
                                                        exportMNT=new ExportVRML();
                                                        fileName=chooser.getSelectedFile().getPath()+".wrl";
                                                        exportMNT.export(mnt, fileName);
                                                    }
                                                if(chooser.getFileFilter()==filter3)
                                                    {
                                                        // Notons que cette opration appelle une mthode de MNT, et n'utilise pas de classe spcifique.
                                                        fileName=chooser.getSelectedFile().getPath()+".mnt";
                                                        DataOutputStream dos = new DataOutputStream (new FileOutputStream(fileName));
                                                        mnt.write(dos);
                                                        dos.close();
							}*/

						String extensionChoisie = (String)(((ExampleFileFilter)(chooser.getFileFilter())).getExtensions().nextElement());
                                                    String fileName=chooser.getSelectedFile().getPath();
                                                    if (!fileName.endsWith(extensionChoisie)){
                                                    	fileName = fileName + "." + extensionChoisie;
                                                    }
                                                    
                                                    
                                                    MntFormat format = GestionnaireMntFormat.donneFormat(extensionChoisie);
						    if (format.support()==MntFormat.IMPORT){
							throw new IOException();
						    }
                                                    
						    format.exportMNT ( mnt,  fileName);
                                                    
                                                getCurrentDocument().setFileName(fileName);
                                                getCurrentDocument().setName(fileName);
                                            }
                                    }
                                catch (IOException ex)
                                    {
                                        System.out.println(ex.getMessage());
                                    }
                            }
                    }
                }
        }
        
    }

/**
 * Superposition des lignes de niveau sur un MNT nouvellement gnr
 * (Un MNT import n'a pas d'information sur les lignes qui ont permis sa gnration)
 */
class ActionSuperpose extends DocumentMNTAction implements SelectDocumentListener

    {
    	
    	protected boolean selected;
        protected DocumentKartoContainer mainWindow;
		JFrame parent;
      
        public ActionSuperpose(JFrame _parent,DocumentMNTContainer w)
        {
            super(w,ResourceManager.get("superpLines"));
            //if((getCurrentDocument().getLignesDeNiveaux())==null) 
            //this.setEnabled(false);
            parent=_parent;
            selected=false;
        }
		
		
        
        public void actionPerformed(ActionEvent e){
            if(getCurrentDocument()!=null){
            	//setSelected(!getSelected());
            	DocumentMNT doc = getCurrentDocument();
            	if(doc.getLignes()!=null){
            		doc.setLdnVisible(this.getSelected());
            	}	
            }                     
        }
        
        private void _documentSelected(Document doc)
        {
        	if(doc==null || ((DocumentMNT)(doc)).getLignes()==null)
                {
                    setEnabled(false);
                    return;
                }
            setEnabled(true);
        }

        public void documentSelected(Document doc)
        {
            //_documentSelected(doc);
            if(doc==null || ((DocumentMNT)(doc)).getLignes()==null)
                {
                    setEnabled(false);
                    return;
                }
            setEnabled(true);
        }
        
    }


