/** 
 *
 * Calque affichant un carroyage. l'espacement entre les lignes est ajustable
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import kartoMNT.karto.math.*;
import java.awt.geom.*;

class Carroyage extends GraphicObject
{

	private Proj projection;
	private double xdim,ydim;
	private double deltax,deltay;
	
	Carroyage()
	{
		deltax=1;
		deltay=1;
		setOpaque(false);
	}


	public void setProj(Proj p)
	{
		if(projection==null)
			{
				projection=p;
				chooseInterval(30);
			}
		else
			{
				projection=p;
			}
		repaint();
	}
	public void setProj(Proj p,double x,double y)
	{
		if(projection==null)
			{
				projection=p;
				chooseInterval(30);
			}
		else
			{
				projection=p;
			}
		setDimension(x,y);
	}

	public void setInterval(double dx,double dy)
	{
		deltax=dx;
		deltay=dy;
		repaint();
	}

	/**
	  * choix automatique d'un intervalle entre les lignes de niveaux
	  * l'intervalle choisi sera une puissance de 10 d'au moins minpixdelta pixels
	  */
	public void chooseInterval(double minpixdelta)
	{
		if(projection==null)
			return;
		double dx,dy;
		dx= (int)((projection.projX(1,0)-projection.projX(0,0))*referential.scalex);
		dy= (int)((projection.projY(1,0)-projection.projY(0,0))*referential.scaley);
		double lx= Math.sqrt(dx*dx+dy*dy);

		dx= (int)((projection.projX(0,1)-projection.projX(0,0))*referential.scalex);
		dy= (int)((projection.projY(0,1)-projection.projY(0,0))*referential.scaley);
		double ly= Math.sqrt(dx*dx+dy*dy);

		deltax= Math.pow(10, Math.ceil(Math.log(minpixdelta/lx)/Math.log(10))  ) ;
		deltay= Math.pow(10, Math.ceil(Math.log(minpixdelta/ly)/Math.log(10))  ) ;
		repaint();
	}

	public void updateReferential()
	{
		Dimension d=new Dimension((int)(xdim*referential.scalex),(int)(ydim*referential.scaley));
		setPreferredSize(d);
		setSize(d);

		super.updateReferential();
		revalidate();
		repaint();

	}


	public void setDimension(double x,double y)
	{
		xdim=x;
		ydim=y;
		updateReferential();
	}

	public void paint(Graphics gr)
	{
		if(projection==null)
			return;

		// on obtient un rectangle englobant
		Point2D.Double p00=projection.pixels2Lambert(new Point2D.Double(0,0));
		Point2D.Double p10=projection.pixels2Lambert(new Point2D.Double(xdim,0));
		Point2D.Double p01=projection.pixels2Lambert(new Point2D.Double(0,ydim));
		Point2D.Double p11=projection.pixels2Lambert(new Point2D.Double(xdim,ydim));

		double xmin,xmax,ymin,ymax;
		xmin=Math.min(Math.min(p00.x,p01.x),Math.min(p10.x,p11.x) );

		ymin=Math.min(Math.min(p00.y,p01.y),Math.min(p10.y,p11.y) );

		xmax=Math.max(Math.max(p00.x,p01.x),Math.max(p10.x,p11.x) );

		ymax=Math.max(Math.max(p00.y,p01.y),Math.max(p10.y,p11.y) );

		xmin= Math.floor(xmin/deltax)*deltax;
		xmax= Math.ceil(xmax/deltax)*deltax;
		ymin= Math.floor(ymin/deltay)*deltay;
		ymax= Math.ceil(ymax/deltay)*deltay;

		// eviter les problemes d'chel dans le carroyage (trop de carrs a tracer)
		if ((xmax-xmin)/deltax>40)
		{
			System.out.println("Erreur sur l'chelle du carroyage ("+(int)((xmax-xmin)/deltax)+" carrs en X). Correction automatique de la taille du carroyage");
			deltax*=10;
			while ((xmax-xmin)/deltax >40)
				deltax*=10;
		} else if((xmax-xmin)/deltax<1)
		{
			System.out.println("Erreur sur l'chelle du carroyage ("+((xmax-xmin)/deltax)+" carrs en X). Correction automatique de la taille du carroyage");
			deltax/=10;
			while ((xmax-xmin)/deltax <1)
				deltax/=10;
		}
		if ((ymax-ymin)/deltay>40)
		{
			System.out.println("Erreur sur l'chelle du carroyage ("+(int)((ymax-ymin)/deltay)+" carrs en Y). Correction automatique de la taille du carroyage");
			deltay*=10;
			while ((ymax-ymin)/deltay >40)
				deltay*=10;
		}else if((ymax-ymin)/deltay<1)
		{
			System.out.println("Erreur sur l'chelle du carroyage ("+((ymax-ymin)/deltay)+" carrs en Y). Correction automatique de la taille du carroyage");
			deltay/=10;
			while ((ymax-ymin)/deltay <1)
				deltay/=10;
		}
		// on trace la grille
		double x,y;
		// lignes verticales

		gr.setColor(new Color(0,255,0));
		for(x=xmin;x<=xmax+.01*deltax;x+=deltax)  //"+.01*deltax" a cause des erreurs d'arrondi potentielles
		{
			gr.drawLine ((int)(projection.projX(x,ymin)*referential.scalex),
						 (int)(projection.projY(x,ymin)*referential.scaley),
						 (int)(projection.projX(x,ymax)*referential.scalex),
						 (int)(projection.projY(x,ymax)*referential.scaley)
						);

		}

		// lignes horizontale
		for(y=ymin;y<=ymax+.01*deltay;y+=deltay)
		{
			gr.drawLine ((int)(projection.projX(xmin,y)*referential.scalex),
						 (int)(projection.projY(xmin,y)*referential.scaley),
						 (int)(projection.projX(xmax,y)*referential.scalex),
						 (int)(projection.projY(xmax,y)*referential.scaley)
						);

		}
	}

};
