/**
 * @author DERHI Roland
 *
 * Programme kartoMNT v2.0
 * Copyright (C) 2003  Roland DERHI & Jean BRESSON
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 * Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 * y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 * ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import java.util.jar.*;

/**
 * Classe chargeant les diffrents formats reconnus par l'application  
*/
public class GestionnaireMntFormat{
	/** Liste contenant l'ensemble des formats reconnus */
	static ArrayList listeFormats = new ArrayList();

	static{
	File dir = new File("kartoMNT");
	// recherche des formats dfinis par l'utilisateur
	if (dir == null) {
		System.err.println(ResourceManager.get("noKart"));	   		 
	}
	else{
		File[] list = dir.listFiles();
		if (list!=null){
		// pour tous les fichiers se trouvant dans kartoMNT
		for (int i = 0; i < list.length; i++) {		
			if (list[i].getName().indexOf('$') == -1 && list[i].getName().endsWith(".class")) {
			// on rcupre le nom de la classe
			String nomClasse = list[i].getName().substring(0,list[i].getName().indexOf('.'));
			

			// s'il s'agit d'un Format
			if ((!nomClasse.equals("MntFormat"))&&(nomClasse.startsWith("MntFormat"))){
				try{
				// on le rajoute  la liste des formats
				listeFormats.add(MNTClassLoader.loadUrl(list[i].toURL().toString(),"kartoMNT") );
				}
				catch(MalformedURLException e){}
			}
			}
			
		}
		}else{
		System.err.println(ResourceManager.get("noFormExtLu"));		
		}

		
		// cration de l'url
		URL url = GestionnaireMntFormat.class.getResource("MntFormat.class");
		if(url != null){
		// Extraction du nom du fichier jar
		String file = url.getFile();
		
		// file contient une String du type
		// file:/D:/Fac/TP plugins/plugin.jar!/tp_plugins/plugins_dessin
		int n = file.indexOf("!");

		// on charge si dans un Jar seulement
		if (n!=-1){
			String nomRepPlugins = file.substring(n + 2,file.lastIndexOf('/'));
			
			try {
			// les classes se trouvent dans un jar
			JarURLConnection jc = (JarURLConnection)url.openConnection();
			JarFile jf = jc.getJarFile();		  
		
			Enumeration e = jf.entries();

			while (e.hasMoreElements()) {
				JarEntry je = (JarEntry)e.nextElement();
				String nomJe = je.getName();
			
				// on ne garde que les entres qui sont sous le rpertoire rep
				if (nomJe.indexOf(nomRepPlugins) == 0) {
				// Le plugin doit tre juste sous le rpertoire
				// et pas  un niveau infrieur dans l'arborescence ;
				// Le nom doit se terminer par .class
				// et ne pas contenir un "$" (pas une classe interne)
				String finNom = nomJe.substring(nomRepPlugins.length() + 1);
				//System.out.println(finNom);
				if (finNom.indexOf("/") == -1 && finNom.indexOf('$') == -1 
					&& finNom.endsWith(".class")) {

					String s = finNom.substring(0, finNom.indexOf(46));
					
					// on vrifie que le fichier commence bien par "Format"
					if(s.length() > 9 && s.startsWith("MntFormat")){

					// Ca correspond bien  une classe de plugin
					// On construit le nom de la classe en enlevant le .class
					// et en remplaant les "/" par des "."
					n = nomJe.lastIndexOf(".class");
					nomJe = nomJe.substring(0, n);
					String nomClasse = nomJe.replace('/', '.');
								
					// on cre un parser
					MntFormat format = creeUneInstance(nomClasse);//"JBourse.serveur."+s);
					//System.out.println(format);
					// on l'ajoute  l'arraylist
					listeFormats.add(format);
				
					}
				}
				}
			}
			}
			catch (IOException err) {
			err.printStackTrace();
			
			}
		}
		
		}
		else{
		System.err.println(ResourceManager.get("noFormatClass"));
		}
	}
		
	

	System.out.println(ResourceManager.get("FormLu"));
	for(Iterator it = listeFormats.iterator() ; it.hasNext() ; ){
		System.out.println(it.next());
	}

	}

	/**
	 * Cre une instance d'un format
	 * @param s Nom du format
	 * @return l'instance du format
	 */
	private static MntFormat creeUneInstance(String s)
	{
		try{
		// on charge la classe
		Class class1 = Class.forName(s);
		MntFormat form = null;
		// on fait appel au constructeur du parser 
		// avec l'objet distant RMI comme parmtre 
		try{
		form = (MntFormat) class1.newInstance();;
		}
		catch(ClassCastException classcastexception){
		System.err.println(ResourceManager.get("Form") + s + ResourceManager.get("Formbad1"));
		return null;
		}
		catch(InstantiationException instantiationexception){
		System.err.println(ResourceManager.get("Form") + s + ResourceManager.get("Formbad2"));
		return null;
		}
		catch(IllegalAccessException illegalaccessexception){
		System.err.println(ResourceManager.get("Form") + s + ResourceManager.get("Formbad3"));
		return null;
		}
		catch(NoClassDefFoundError noclassdeffounderror){
		System.err.println(ResourceManager.get("Form") + s + ResourceManager.get("Formbad4"));
		return null;
		}
		
			
		
		
		return form;
	}		
	catch(ClassNotFoundException classnotfoundexception){
		System.err.println(ResourceManager.get("Form") + s + ResourceManager.get("Formbad5"));
	}
		return null;
	}

	
	/**
	   Mthode retournant un MntFormat  partir de son nom du fichier ou de son extension
	   @param name l'extension ou le nom du fichier
	*/
	public static MntFormat donneFormat(String name){
		String extension = null;
		// on rcupre l'extension
		if (name.lastIndexOf('.')==-1)
			extension = name;
		else
			extension = name.substring(name.lastIndexOf('.')+1);
	
		// pour tous les lments de la liste des formats grs
		for(Iterator it = listeFormats.iterator() ; it.hasNext() ; ){
			MntFormat format = null;
			try{
				format = (MntFormat)it.next();
			}
			catch(ClassCastException e){
				continue;
			}
			// on vrifie si le format correspond  l'extension
			if (format.getExtension().equalsIgnoreCase(extension)){
				return format;
			}
		}
		return null;
	}

	/**
	 * Mthode initialisant un JFileChooser avec l'ensemble des formats disponibles
	 * @param jf le JFileChooser  initialiser
	 * @param mode mode d'initialisation (MntFormat.EXPORT ou MntFormat.IMPORT)
	*/
	public static void initFilter(JFileChooser jf,int mode){
		ExampleFileFilter filtreKartoMNT=null;
		String descriptionKartoMNT= new MntFormatMNT().getDescription();
		for(Iterator it = listeFormats.iterator() ; it.hasNext() ; ){
			MntFormat format = null;
			try{
				format = (MntFormat)it.next();
			}
			catch(ClassCastException e){
				continue;
			}
				
			if ((mode==format.support())||(format.support()==MntFormat.EXPORT+MntFormat.IMPORT)){			
				ExampleFileFilter filter = new ExampleFileFilter();
				filter.addExtension(format.getExtension());
				filter.setDescription(format.getDescription());
				if (format.getDescription().equals(descriptionKartoMNT))
					filtreKartoMNT=filter;
				else
					jf.addChoosableFileFilter(filter);
			}
		}
		// inserer  la fin pour qu'il soit slectionn par dfaut.
		if (filtreKartoMNT!=null)
			jf.addChoosableFileFilter(filtreKartoMNT);
	}
}
