/** 
 * Classe de base de l'ensemble des Calques
 * On peut dfinir un repre dans lequel est plac le calque et une position
 * Le repre est utile si plusieurs calques utilisent le mme repre et qu'on applique un
 * changement d'chelle (zoom de l'ensemble des calques)
 *
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import javax.swing.*;
import java.awt.*;



abstract class GraphicObject extends JPanel {
    
    protected double posx,posy;
    protected ObjectReferential referential;
    protected int center_dx,center_dy;
    protected double alpha;
    
    GraphicObject(){
        super();
        referential=new ObjectReferential();
        posx=0;
        posy=0;
        center_dx=0;
        center_dy=0;
    }

    /** 
     * definit la position de l'objet
     * les coordonnes sont dfinies dans le rfrentiel de l'objet (diffrent des coordones cran)
     */
    public void setObjectPosition(double x,double y){
        posx=x;
        posy=y;
        updateReferential();
	}


    public double getPosY(){
        return posy;
    } 
    
    public double getPosX(){
        return posx;
    }

	//screenx=posx*referential.scalex+referential.posx
    //screeny=posy*referential.scaley+referential.posy

    public void setObjectPositionScreenCoords(int screenx,int screeny){
        posx=(screenx-referential.posx)/referential.scalex;
        posy=(screeny-referential.posy)/referential.scaley;

        setLocation(screenx-center_dx,screeny-center_dy);
        revalidate();
        repaint();
    }

    public int getScreenPosX(){
        return getLocation().x+center_dx;
    }

    public int getScreenPosY(){
        return getLocation().y+center_dy;
    }

    public Point getScreenPos(){
        return getLocation();
    }

    /** 
     * definit le referentiel de l'objet 
     */
    public void setReferential(ObjectReferential r){
        referential=r;
        updateReferential();
    }

    /* obtient le referentiel de l'objet
     * si le referentiel retourne est modifi, if faut appeler updateReferential 
     * pour que les changements soient visibles
     */
    public ObjectReferential getReferential(){
        return referential;
    }

    public double canvas2ObjX(double x){
        return (x-center_dx)/referential.scalex;
    }

    public double canvas2ObjY(double y){
        return (y-center_dx)/referential.scaley;
    }

    public double obj2CanvasX(double x){
        return x*referential.scalex+center_dx;
    }

    public double obj2CanvasY(double y){
        return y*referential.scaley+center_dy;
    }

    public double getScaleX(){
        return referential.scalex;
    }
    
    public double getScaleY(){
        return referential.scaley;
    }

    public void updateReferential(){
        setLocation((int)(posx * referential.scalex + referential.posx - center_dx), (int)(posy*referential.scaley+referential.posy-center_dy));
        revalidate();
        repaint();
    }

    void setAlpha(double alpha){}

    double getAlpha(){
        return alpha;
    }

}

