/** 
 * Palette de couleur interpole
 * On definit un ensemble de couleurs en diffrentes positions. La couleur tant interpole entre les diffrentes
 * positions
 *
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import java.awt.*;
import java.util.*;
import javax.swing.event.*;




class InterpolatedPalette
    {

		protected Vector colors;
        protected Vector listeners;
        protected boolean sorted;
		
        public InterpolatedPalette()
        {
            colors=new Vector();
            listeners=new Vector();
            sorted=true;
            //changeEvent=new ChangeEvent(this);
        }


        void addPaletteListener(InterpolatedPaletteListener l)
        {
            listeners.add(l);
        }

        void removePaletteListener(InterpolatedPaletteListener l)
        {
            listeners.remove(l);
        }


        protected void notifyColorAdded(int i)
        {
            ListIterator it=listeners.listIterator();
            while(it.hasNext())
                {
                    ((InterpolatedPaletteListener)it.next()).colorAdded(this,i);
                }

        }

        protected void notifyColorRemoved(int i)
        {
            ListIterator it=listeners.listIterator();
            while(it.hasNext())
                {
                    ((InterpolatedPaletteListener)it.next()).colorRemoved(this,i);
                }

        }

        /*protected void notifyColorModified(int i)
        {
        ListIterator it=listeners.listIterator();
        while(it.hasNext())
            { ((InterpolatedPaletteListener)it.next()).colorModified(this,i);
            }

    }*/

        protected void notifyColorMoved(int i)
        {
            ListIterator it=listeners.listIterator();
            while(it.hasNext())
                {
                    ((InterpolatedPaletteListener)it.next()).colorMoved(this,i);
                }
        }

        protected void notifyColorChanged(int i)
        {
            ListIterator it=listeners.listIterator();
            while(it.hasNext())
                {
                    ((InterpolatedPaletteListener)it.next()).colorChanged(this,i);
                }

        }


        public void setSorted(boolean b)
        {
            sorted=b;
            if(sorted)
                Collections.sort(colors);
        }

        public boolean isSorted()
        {
            return sorted;
        }

        public void addColor(double x,Color color)
        {
            ColorElement c;
            colors.add(c=new ColorElement(x,color));
            if(sorted)
                Collections.sort(colors);
            int i=colors.indexOf(c);
            notifyColorAdded(i);
        }

        public int getNbColor()
        {
            return colors.size();
        }

        public ListIterator getListIterator()
        {
            return colors.listIterator();
        }

        public void removeColor(int i)
        {
            colors.remove(i);
            notifyColorRemoved(i);
        }

        public Color getColor(int i)
        {
            return ((ColorElement)colors.elementAt(i)).color;
        }

        public double getPosition(int i)
        {
            return ((ColorElement)colors.elementAt(i)).x;
        }

        public void setColor(int i,Color c)
        {
            ((ColorElement)colors.elementAt(i)).color=c;
            notifyColorChanged(i);

        }


        double getMinPos()
        {
            if(colors.size()==0)
                {
                    return 0;
                }
            return ((ColorElement)colors.elementAt(0)).x;
        }

        double getMaxPos()
        {
            if(colors.size()==0)
                {
                    return 0;
                }
            return ((ColorElement)colors.elementAt(colors.size()-1)).x;
        }


        public void setPosition(int lastindice,double x)
        {
            ColorElement c;
            c=(ColorElement)colors.elementAt(lastindice);
            c.x=x;
            notifyColorMoved(lastindice);

            if(sorted)
                Collections.sort(colors);
            //int j=colors.indexOf(c);

        }



        public Color[] renderPalette(double xmin,double xmax,int nbcolor)
        {
            Color[] pal=new Color[Math.max(nbcolor,0)];
            ListIterator it=colors.listIterator();

            int i;
            if(!it.hasNext())
                {
                    for(i=0;i<nbcolor;i++)
                        {
                            pal[i]=Color.black;
                        }
                    return pal;
                }

            ColorElement color1,color2;
            double delta=(xmax-xmin)/(nbcolor-1);
            color2=(ColorElement)it.next();
            i=0;
            double x=xmin;
            while((i<nbcolor)&&(x<color2.x))
                {
                    pal[i]=color2.color;
                    i++;
                    x+=delta;
                }
            double a;
            int r,g,b;

            while((it.hasNext())&&(i<nbcolor))
                {
                    color1=color2;
                    color2=(ColorElement)it.next();

                    while((i<nbcolor)&&(x<color2.x))
                        {
                            a=(x-color1.x)/(color2.x-color1.x);
                            r=(int)(a*color2.color.getRed()+(1-a)*color1.color.getRed());
                            g=(int)(a*color2.color.getGreen()+(1-a)*color1.color.getGreen());
                            b=(int)(a*color2.color.getBlue()+(1-a)*color1.color.getBlue());
                            pal[i]=new Color(r,g,b);
                            i++;
                            x+=delta;
                        }
                }

            while(i<nbcolor)
                {
                    pal[i]=color2.color;
                    i++;
                    x+=delta;
                }
            return pal;

        }




        public class ColorElement implements Comparable
            {
            	
            	public double x;
                public Color color;
            	
                public ColorElement(double _x,Color _color)
                {
                    x=_x;
                    color=_color;
                }
                

                public int compareTo(Object o)
                {
                    if (x<((ColorElement)o).x)
                        return -1;
                    else if (x>((ColorElement)o).x)
                        return 1;
                    else
                        return 0;
                }

            }
        
    }


