/**
 * @author Jean BRESSON
 *
 * Programme kartoMNT v2.0
 * Copyright (C) 2003  Roland DERHI & Jean BRESSON
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 * Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 * y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 * ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.util.Vector;

/**
 * Permet de gnrer ou MNT  partir d'un ou de plusieurs autres et de caracteristiques donnes
 */
public class MNTGenerateur{
	
	
	private float x0, y0;		// position du MNT
	private float pasX, pasY;	// pas d'chantillonnage
	private int w , h ;			// dimensions
	
	private Vector vectMNT;		// MNT utiliss pour la gnration			
	
	
	/**
	 * Constructeur
	 * @param width Largeur du MNT  gnrer
	 * @param height Hauteur du MNT  gnrer
	 * @param x1 position en X du MNT  gnrer
	 * @param y1 position en Y du MNT  gnrer
	 * @param pX pas d'chantillonnage en X du MNT  gnrer
	 * @param pY pas d'chantillonnage en Y du MNT  gnrer
	 * @param vec Vecteur contenant les MNT utiliss
	 */
	public MNTGenerateur(int width, int height, 
						  float x1,float  y1, float pX, float pY, Vector vec){
	 	w = width;
	 	h = height;
	 	x0 = x1;
	 	y0 = y1;
	 	pasX = pX;
	 	pasY = pY;
	 	vectMNT = vec;
	}
	
	/**
	 * Gnre et renvoie un nouveau MNT
	 * L'altitude en chaque point est calcule par interpolation sur les points 
	 * voisins d'un des MNT.
	 * @return le MNT gnr
	 */
	public MNT genererMNT(){
		MNT newMNT = new MNT(w, h, x0, y0, pasX, pasY);
		// Parcours du nouveau MNT
		for(int i=0; i< h; i++){
			for(int j=0; j<w; j++){
				// determiner les coordonnes x,y du point
				float x = x0+(j*pasX);
				float y = y0+(i*pasY);				
			
				// chercher dans quel MNT on le trouve
				MNT mnt= new MNT();
				boolean found=false;
				int m;
				for(m = 0; m< vectMNT.size() && !found; m++){
					MNT mntTemp = (MNT) vectMNT.elementAt(m);	
					if(mntTemp.getX1()<=x && mntTemp.getY1()<=y && mntTemp.getX2()>x && mntTemp.getY2()>y){ 
						mnt = mntTemp;
						found = true;
					}
				}
						
				// Si on n'a pas d'info sur ce point: alt=0
				if(!found){
					newMNT.setValue(j,i,0);
					//System.out.println(j + " " + i +" ** ");
				} else {
					// Le point est dans le MNT mnt
							
					// determiner les quatre points autour;
					// le point sup-gauche
					boolean bX=false;
					boolean bY=false;
							
					// pas du MNT courant
					float pX=mnt.getPasX();
					float pY=mnt.getPasY();
							
					// position en X et Y du point par rapport au MNT
					float posX = (x - mnt.getX1()); // position en X du point par rapport au MNT
					float posY = (y - mnt.getY1()); 
							
					// numero du pixel correspondant
					int pixX = (int) Math.floor(posX/pX);
					int pixY = (int) Math.floor(posY/pY);
						
					// Si la position en X ou Y correspond  une coordonne entire du MNT
					// ou  la dernire donne connue: il ne faut pas interpoler dans cette direction.
					if(posX==pixX*pX || pixX==mnt.getXMax()-1) bX = true; 
					if(posY==pixY*pY || pixY==mnt.getYMax()-1) bY = true; 
						
					float altNO=-1;
					float altNE=-1; 
					float altSO=-1; 	
					float altSE=-1;
					double dNO=-1;
					double dNE=-1;
					double dSO=-1;
					double dSE=-1;
							 
					try{
						// calcul des points autour et des distances  ces points
						// Si on est sur une abscisse ou ordonne entire on ne va pas chercher les points (on risquerait de sortir du MNT)							
						altNO = mnt.getValue(pixX,pixY);
						dNO = Math.sqrt(Math.pow(posX-pixX*pX,2)+Math.pow(posY-pixY*pY,2));
								
						if(!bX){
							 altNE = mnt.getValue(pixX+1,pixY);
							 dNE = Math.sqrt(Math.pow(posX-(pixX+1)*pX,2)+Math.pow(posY-pixY*pY,2));
						}
						if(!bY){
							 altSO = mnt.getValue(pixX,pixY+1);
							 dSO = Math.sqrt(Math.pow(posX-pixX*pX,2)+Math.pow( posY-(pixY+1)*pY,2));
						}
						if(!bX && !bY){
							 altSE = mnt.getValue(pixX+1,pixY+1);
							 dSE = Math.sqrt(Math.pow(posX-(pixX+1)*pX,2)+Math.pow(posY-(pixY+1)*pY,2));
						}
								
						// calcul de l'altitude (interpolation sur 1, 2 ou 4 points en fonction des cas prcdents)
						float alt;
						int n = 2;	 // pondration de la distance
						dNO=Math.pow(dNO,-n);
						dNE=Math.pow(dNE,-n);
						dSO=Math.pow(dSO,-n);
						dSE=Math.pow(dSE,-n);
						
						if(bX && bY) alt=altNO;
						else if(bX) alt = (float)((altNO*dNO+altSO*dSO)/(dNO+dSO));
						else if(bY) alt = (float)((altNO*dNO+altNE*dNE)/(dNO+dNE));
						else alt = (float)((altNO*dNO+altNE*dNE+altSO*dSO+altSE*dSE)/(dNO+dNE+dSO+dSE));
								
						newMNT.setValue(j,i,alt);
								
					} catch (Exception e){
						newMNT.setValue(j,i,10000);
						System.out.println("mnt "+ m + "  " + j + " " + i + " -- ["+pixX+" "+pixY+"] Error!" );
						System.out.println("bX: "+bX+"    bY: "+bY);
						System.out.println("MNT: ("+mnt.getX1()+","+mnt.getY1()+")->("+mnt.getX2()+","+mnt.getY2()+") ["+mnt.getXMax()+"x"+mnt.getYMax()+"]  pasX="+mnt.getPasX()+" pasY="+mnt.getPasY()+"");
					}
				}						
			}
		}
		
		return newMNT;
	}
	
	
}
	