/** 
 * Fonction utilitaires
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 * module Util.java 
 *
 * Programme kartoMNT : cartographie 2D sur cartes numriques
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import kartoMNT.karto.math.*;
import java.awt.geom.*;

public class MathUtil
    {
        public static Rectangle2 getEnglobingRect(Proj projection,double xmin,double ymin,double xmax,double ymax)
        {
            Point2D.Double p00=projection.pixels2Lambert(new Point2D.Double(xmin,ymin));
            Point2D.Double p10=projection.pixels2Lambert(new Point2D.Double(xmax,ymin));
            Point2D.Double p01=projection.pixels2Lambert(new Point2D.Double(xmin,ymax));
            Point2D.Double p11=projection.pixels2Lambert(new Point2D.Double(xmax,ymax));


            Rectangle2 rect=new Rectangle2();
            rect.x1=Math.min(Math.min(p00.x,p01.x),Math.min(p10.x,p11.x) );
            rect.y1=Math.min(Math.min(p00.y,p01.y),Math.min(p10.y,p11.y) );
            rect.x2=Math.max(Math.max(p00.x,p01.x),Math.max(p10.x,p11.x) );
            rect.y2=Math.max(Math.max(p00.y,p01.y),Math.max(p10.y,p11.y) );
            return rect;
        }


        /* [M0, M1]
           [M2, M3] : matrice 2*2
           pasX: pas en X
           pasY: pas en Y

          Valeur retourne
             La fonction considre la matrice
             A= [M0*pasX, M1*pasY] 
                   [M2*pasX, M3*pasY]
             et retourne la valeur minimale de ||A*X|| sous la contrainte que ||X||=1
             
            Cela correspond au dplacement minimal dans l'espace d'arrive quand on se dplace d'une
            unit dans l'espace de dpart
        */
        static double depmin(double M0,double M1,double M2,double M3,double pasX,double pasY)
        {

            //calcul de la plus petite valeur propre de Mt * M
            double v;
            double[] mat0=new double[4];
            mat0[0]=M0*pasX;
            mat0[1]=M1*pasY;
            mat0[2]=M2*pasX;
            mat0[3]=M3*pasY;

            double[] mat=new double[4];
            mat[0]=mat0[0]*mat0[0]+mat0[1]*mat0[1];
            mat[1]=mat0[0]*mat0[2]+mat0[1]*mat0[3];
            mat[2]=mat0[2]*mat0[0]+mat0[3]*mat0[1];
            mat[3]=mat0[2]*mat0[2]+mat0[3]*mat0[3];

            double b=-mat[0]-mat[3];
            double c=mat[0]*mat[3]-mat[1]*mat[2];
            double delta=b*b-4*c;
            if(delta>=0)
                {
                    v=.5*(-b-Math.sqrt(delta));
                }
            else
                {
                    v=-.5*b;
                } // ne devrait jamais se produire
            double depmin;
            if(v>=0)
                depmin=Math.sqrt(v);
            else
                depmin=0; // ne devrait jamais se produire
            return depmin;
        }

        /* [M0, M1]
           [M2, M3] : matrice 2*2
           pasX: pas en X
           pasY: pas en Y

          Valeur retourne
             la fonction considre la matrice
             A= [M0*pasX, M1*pasY] 
                   [M2*pasX, M3*pasY]
             et retourne la valeur maximale de ||A*X|| sous la contrainte que ||X||=1
             
            Cela correspond au dplacement minimal dans l'espace d'arrive quand on se dplace d'une
            unit dans l'espace de dpart
         */
        static double depmax(double M0,double M1,double M2,double M3,double pasX,double pasY)
        {

            //calcul de la plus petite valeur propre de Mt * M
            double v;
            double[] mat0=new double[4];
            mat0[0]=M0*pasX;
            mat0[1]=M1*pasY;
            mat0[2]=M2*pasX;
            mat0[3]=M3*pasY;

            double[] mat=new double[4];
            mat[0]=mat0[0]*mat0[0]+mat0[1]*mat0[1];
            mat[1]=mat0[0]*mat0[2]+mat0[1]*mat0[3];
            mat[2]=mat0[2]*mat0[0]+mat0[3]*mat0[1];
            mat[3]=mat0[2]*mat0[2]+mat0[3]*mat0[3];

            double b=-mat[0]-mat[3];
            double c=mat[0]*mat[3]-mat[1]*mat[2];
            double delta=b*b-4*c;
            if(delta>=0)
                {
                    v=.5*(-b+Math.sqrt(delta));
                }
            else
                {
                    v=-.5*b;
                } // ne devrait jamais se produire
            double depmax;
            if(v>=0)
                depmax=Math.sqrt(v);
            else
                depmax=0; // ne devrait jamais se produire
            return depmax;
        }


    }
