package kartoMNT;

import java.io.*;
import javax.swing.*;

/**
   Format de MNT grant le format Leveller (.ter)
*/
public class MntFormatLeveller implements MntFormat{

    /** Constructeur */
    public MntFormatLeveller(){}

    /**
       Retourne l'extension gre par le format
       @return "ter"
    */
    public String getExtension(){
	return "ter";
    }

    /**
       Retourne une description du format gr
       @return description du format gr
    */
    public String getDescription(){
	return "Leveller";
    }

    /**
     * Mode de support du format
     * @return MntFormat.EXPORT (le format ne gre que l'exportation)
    */
    public int support(){
	return MntFormat.EXPORT;
    }

    /**
       Fonction d'exportation du format
       @param MNT le mnt  exporter
       @param filename nom du fichier o sera sauvegarder le MNT
       @throws  IOException en cas de problme d'criture du fichier
    */
    public void exportMNT (MNT mnt, String filename) throws IOException{
	
	Object[] options = {"Version 4",
                                                        "Version 5"};
	int value = JOptionPane.showOptionDialog(null,
					     "Quel version de TER prfrez-vous ?\n(La verion 4 perd les pas en X et Y)",
					     "Question",
					     JOptionPane.YES_NO_OPTION,
					     JOptionPane.QUESTION_MESSAGE,
					     null,     //don't use a custom Icon
					     options,  //the titles of buttons
					     options[0]); //default button title

	boolean version5=true;
	if (value == JOptionPane.YES_OPTION) {
	    version5=false;
	}

	FileOutputStream fos = new FileOutputStream(filename);
	LittleEndianOutputStream dos = new LittleEndianOutputStream(fos);
	int i,j;

	// magic
	dos.writeBytes("trrn");
	// version	
	if (version5){
	    dos.writeByte(5);
	}
	else{
	    dos.writeByte(4);
	}

	// width
	dos.writeByte(4);
	dos.writeBytes("hf_w");
	dos.writeInt(4);
	dos.writeInt(mnt.getXMax());

	// breadth
	dos.writeByte(4);
	dos.writeBytes("hf_b");
	dos.writeInt(4);
	dos.writeInt(mnt.getYMax());

	if (version5){
	    // Horizontal Spacing
	    dos.writeByte(9);
	    dos.writeBytes("grid_spcX");
	    dos.writeInt(4);
	    dos.writeFloat(mnt.getPasX());
	    
	    // Breadthwise Spacing
	    dos.writeByte(9);
	    dos.writeBytes("grid_spcZ");
	    dos.writeInt(4);
	    dos.writeFloat(mnt.getPasY());
	    
	    // Horz. offset, in heightfield pixels
	    /*dos.writeByte(7);
	      dos.writeBytes("grid_ox");
	      dos.writeInt(4);
	      dos.writeFloat(mnt.getX1());*/
	    
	    // Vert. offset, in heightfield pixels
	    /*dos.writeByte(7);
	      dos.writeBytes("grid_oz");
	      dos.writeInt(4);
	      dos.writeFloat(mnt.getY1());*/
	}

	// data
	dos.writeByte(7);
	dos.writeBytes("hf_data");
	dos.writeInt(mnt.getXMax()*mnt.getYMax()*4);
	
	
	for (j = 0; j < mnt.getYMax(); j++)
	    for (i = 0; i < mnt.getXMax(); i++)
		{
		    dos.writeInt((int)mnt.getValue(i, j)*65536);
		}
	dos.close();
	
    }

    /**
       Fonction d'importation du format       
       @param filename nom du fichier o sera sauvegarder le MNT
       @return le MNT import
       @throws  IOException en cas de problme de lecture du fichier
    */
    public MNT importMNT(String filename) throws IOException{
	return null;
    }

    /**
       Mthode toString
       @return extension gr par le format
    */
    public String toString(){
	return "Format "+getExtension();
    }

}
