/**
 * @author DERHI Roland
 *
 * Programme kartoMNT v2.0
 * Copyright (C) 2003  Roland DERHI & Jean BRESSON
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 * Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 * Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 * y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 * ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;

import java.io.*;

/**
   Format de MNT grant les .mnt
*/
public class MntFormatMNT implements MntFormat{

    /** Constructeur */
    public MntFormatMNT(){}

    /**
       Retourne l'extension gre par le format
       @return "mnt"
    */
    public String getExtension(){
	return "mnt";
    }

    /**
       Retourne une description du format gr
       @return description du format gr
    */
    public String getDescription(){
	return "Format KartoMNT binaire";
    }

    /**
     * Mode de support du format
     * @return MntFormat.EXPORT+MntFormat.IMPORT (le format gre l'importation et l'exportation)
    */
    public int support(){
	return MntFormat.EXPORT+MntFormat.IMPORT;
    }

    /**
       Fonction d'exportation du format
       @param MNT le mnt  exporter
       @param filename nom du fichier o sera sauvegarder le MNT
       @throws  IOException en cas de problme d'criture du fichier
    */
    public void exportMNT (MNT mnt, String filename) throws IOException{

	FileOutputStream fos = new FileOutputStream(filename);
	DataOutputStream dos = new DataOutputStream(fos);	

	int i, j;
	// xmax
	dos.writeInt(mnt.getXMax());
	// ymax
	dos.writeInt(mnt.getYMax());
	// coordonne du point en haut  gauche (0,0)
	dos.writeFloat(mnt.getX1());
	dos.writeFloat(mnt.getY1());
	// pas
	dos.writeFloat(mnt.getPasX());
	dos.writeFloat(mnt.getPasY());
	// le mnt
	for (i = 0; i < mnt.getXMax(); i++)
	    for (j = 0; j < mnt.getYMax(); j++)
		{
		    dos.writeFloat(mnt.getValue(i, j));
		}
	dos.close();
    }

    /**
       Fonction d'importation du format       
       @param filename nom du fichier o sera sauvegarder le MNT
       @return le MNT import
       @throws  IOException en cas de problme de lecture du fichier
    */
    public MNT importMNT(String filename) throws IOException{

	FileInputStream fis = new FileInputStream(filename);
	DataInputStream dis = new DataInputStream(fis);	

	int i, j;
	// xmax et ymax
	int xmax=dis.readInt();
	int ymax=dis.readInt();
	float[] data=new float[xmax*ymax];
	// coordonnes du point en haut  gauche (0,0)
	float X1=dis.readFloat();
	float Y1=dis.readFloat();
	// pas en X et Y
	float pasX=dis.readFloat();
	float pasY=dis.readFloat();

	// le mnt
	MNT mnt = new MNT(xmax, ymax, X1, Y1, pasX, pasY);

	for (i = 0; i < xmax; i++)
	    for (j = 0; j < ymax; j++)
		{
		    data[i + xmax * j] = dis.readFloat();
		}

	mnt.setFloatData(data);

	return mnt;
    }

    /**
       Mthode toString
       @return extension gr par le format
    */
    public String toString(){
	return "Format "+getExtension();
    }
    
}
