/** 
 * PointCalibrationModel contient les donnes d'un point de calibration: coordonnes terrain et image
 *  permet de dispatcher les modification a un ensemble de listeners (par exemple  un objet PointCalibration
 *  qui permet de visualiser graphiquement le point, ou  une boite de dialogue pour mettre  jour les coordonnes)
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import java.util.*;
import java.awt.geom.*;
import java.io.*;

class PointCalibrationModel
    {
        PointCalibrationModel()
        {
            listeners=new Vector();
            selected=false;
        }

        public void write(DataOutputStream dos) throws IOException
            {
                dos.writeDouble(ximage);
                dos.writeDouble(yimage);
                dos.writeDouble(x);
                dos.writeDouble(y);
            }

        public void write2(BufferedWriter dos) throws IOException
            {
                dos.write(""+ximage);
                dos.write(" "+yimage);
                dos.write(" "+x);
                dos.write(" "+y);
            }

        public void read(DataInputStream dis) throws IOException
            {
                ximage=dis.readDouble();
                yimage=dis.readDouble();
                x=dis.readDouble();
                y=dis.readDouble();
            }


        void addListener(PointCalibrationListener l)
        {
            listeners.add(l);
        }

        void removeListener(PointCalibrationListener l)
        {
            listeners.remove(l);
        }

        void clearListener(PointCalibrationListener l)
        {
            listeners.clear();
        }

        protected void notifyListeners()
        {
            ListIterator it=listeners.listIterator();
            while(it.hasNext())
                {
                    ((PointCalibrationListener)it.next()).pointCalibrationChanged(this);
                }
        }

        protected void notifyListenersSelect(boolean b)
        {
            ListIterator it=listeners.listIterator();
            while(it.hasNext())
                {
                    ((PointCalibrationListener)it.next()).pointCalibrationSelected(b);
                }
        }

        void setPointCalibrationSet(PointCalibrationSet p)
        {
            pointCalibrationSet=p;
        }

        PointCalibrationSet getPointCalibrationSet()
        {
            return pointCalibrationSet;
        }

        void select()
        {
            selected=true;
            if(pointCalibrationSet!=null)
                pointCalibrationSet.select(this);
        }

        void setCoords(double _x,double _y)
        {
            x=_x;
            y=_y;
            notifyListeners();
        }
        Point2D.Double getCoords()
        {
            return new Point2D.Double(x,y);
        }
        double getX()
        {
            return x;
        }
        double getY()
        {
            return y;
        }

        void setImageCoords(double x,double y)
        {
            ximage=x;
            yimage=y;
            notifyListeners();
        }

        Point2D.Double getImageCoords()
        {
            return new Point2D.Double(ximage,yimage);
        }

        double getImageX()
        {
            return ximage;
        }

        double getImageY()
        {
            return yimage;
        }

        public void pointCalibrationSelected(PointCalibrationModel p)
        {
            selected=(p==this);
            notifyListenersSelect(selected);
        }

        public boolean getSelected()
        {
            return selected;
        }

        private double x,y;
        private double ximage,yimage;

        private Vector listeners;
        boolean selected;

        PointCalibrationSet pointCalibrationSet;
    };
