/** 
 * Calque affichant la selction d'une rgion rectangulaire (si la matrice de calibration est une rotation)
 *  cet objet est utilis par l'objet ToolSelectionRegionMNT
 * 
 * @author HEIT Sylvestre & Coustillac Laurent
 
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import kartoMNT.karto.math.*;
import java.awt.geom.*;

class SelectionRegion extends GraphicObject
    {
		
		private Proj projection;
        private double x0,y0,x1,y1;
        private double px0,py0,px1,py1;
        private double deltax,deltay;
        private double xdim,ydim;
        private boolean enabled,afficherLesPoints;

        private double echdx,echdy;
        private Rectangle2 rect;
		
        SelectionRegion()
        {
            setOpaque(false);
            enabled=true;
            setDimension(100,100);
            echdx=0;
            echdy=0;
            afficherLesPoints=false;
        }

        public void setEnabled(boolean b)
        {
            enabled=b;
        }

        public void afficherLesPoints(boolean b)
        {
            if(b!=afficherLesPoints)
                {
                    afficherLesPoints=b;
                    //System.out.println("mode " + mode);
                    repaint();
                }
        }

        public void setProj(Proj p)
        {
            projection=p;
            computeMode();
            if(projection!=null)
                rect=MathUtil.getEnglobingRect(projection,0,0,xdim,ydim);
            repaint();
        }

        private static final int NONE=0;
        private static final int FILL=1;
        private static final int LINEX=2;
        private static final int LINEY=3;
        private static final int POINTS=4;
        private int mode;
        private boolean transparent;

        private void computeMode()
        {
            mode=NONE;
            if((projection==null) || (echdx==0)|| (echdy==0))
                return;
            mode=POINTS;
            double[] pr=projection.getProj();
            double depmin=MathUtil.depmin(pr[0]*referential.scalex,pr[1]*referential.scalex,
                                          pr[3]*referential.scaley,pr[4]*referential.scaley,
                                          echdx,echdy);
            if(depmin>=2)
                {
                    transparent=false;
                }
            else
                {
                    transparent=true;
                }
            if(MathUtil.depmax(pr[0]*referential.scalex,pr[1]*referential.scalex,
                               pr[3]*referential.scaley,pr[4]*referential.scaley,echdx,echdy)< Math.sqrt(2.0)/2.0*.999 )
                {
                    mode=FILL;
                }
            else
                {
                    double dx,dy,l,l2;
                    dx=pr[0]*echdx*referential.scalex;
                    dy=pr[3]*echdx*referential.scaley;
                    l=Math.sqrt(dx*dx+dy*dy);
                    if(l<1.0)
                        {
                            mode=LINEX;
                        }

                    dx=pr[1]*echdy*referential.scalex;
                    dy=pr[4]*echdy*referential.scaley;
                    l2=Math.sqrt(dx*dx+dy*dy);
                    if((l2<1.0)&&(l2<l))
                        {
                            mode=LINEY;
                        }


                }

        }

        public void setEchantillonDelta(double dx,double dy)
        {
            //echdx=dy;
            //echdy=dx;
        	// corrig : les pas dans l'affichage des points taient inverss
        	echdx=dx;
            echdy=dy;
            computeMode();
            repaint();
        }

        public void setRegion(double xx0,double yy0,double xx1,double yy1)
        {
            //if(projection==null) {x0=xx0;y0=yy0;x1=xx1;y1=yy1;return;}
            if(projection==null)
                {
                    return;
                }

            Point2D.Double p0=projection.pixels2Lambert(Math.min(xx0,xx1),Math.min(yy0,yy1));
            Point2D.Double p1=projection.pixels2Lambert(Math.max(xx1,xx0),Math.max(yy1,yy0));
            px0=p0.x;
            py0=p0.y;

            px1=p1.x;
            py1=p1.y;

            repaint();
        }


        public void setRegionLambert(double xx0,double yy0,double xx1,double yy1)
        {
            px0=Math.min(xx0,xx1);
            py0=Math.min(yy0,yy1);

            px1=Math.max(xx0,xx1);
            py1=Math.max(yy0,yy1);

            repaint();
        }


        public void setProj(Proj p,double x,double y)
        {
            projection=p;
            if(projection!=null)
                rect=MathUtil.getEnglobingRect(projection,0,0,xdim,ydim);
            computeMode();
            setDimension(x,y);
        }

        public void setInterval(double dx,double dy)
        {
            deltax=dx;
            deltay=dy;
            repaint();
        }

        public void updateReferential()
        {
            Dimension d=new Dimension(round(xdim*referential.scalex),round(ydim*referential.scaley));
            setPreferredSize(d);
            setSize(d);

            super.updateReferential();
            computeMode();
            revalidate();
            repaint();

        }

        // arrondi  l'entier le plus proche
        protected int round(double x)
        {
            return (int)(x+.5);
        }

        public void setDimension(double x,double y)
        {
            xdim=x;
            ydim=y;
            if(projection!=null)
                rect=MathUtil.getEnglobingRect(projection,0,0,xdim,ydim);
            updateReferential();
        }

        public void paint(Graphics gr)
        {
            if((!enabled)||(projection==null))
                return;

            double x00,x01,x10,x11;
            double y00,y01,y10,y11;

            /*Point2D.Double p0=projection.pixels2Lambert(x0,y0);
            Point2D.Double p1=projection.pixels2Lambert(x1,y1);*/

            x00=projection.projX(px0,py0)*referential.scalex;
            x10=projection.projX(px1,py0)*referential.scalex;
            x01=projection.projX(px0,py1)*referential.scalex;
            x11=projection.projX(px1,py1)*referential.scalex;

            y00=projection.projY(px0,py0)*referential.scaley;
            y10=projection.projY(px1,py0)*referential.scaley;
            y01=projection.projY(px0,py1)*referential.scaley;
            y11=projection.projY(px1,py1)*referential.scaley;

            gr.setColor(new Color(0,127,255));

            gr.drawLine(round(x00),round(y00),round(x01),round(y01));
            gr.drawLine(round(x01),round(y01),round(x11),round(y11));
            gr.drawLine(round(x11),round(y11),round(x10),round(y10));
            gr.drawLine(round(x10),round(y10),round(x00),round(y00));

            if(afficherLesPoints)
                {
                    if(transparent)
                        gr.setColor(new Color(0,127,255,100));
                    
                    if( mode==POINTS ){
                        int xx,yy;
                        double x,y;
                        double xa,xb,ya,yb;
                        xa=Math.max(Math.min(rect.x2,px0),rect.x1);
                        xb=Math.max(Math.min(rect.x2,px1),rect.x1);
                        ya=Math.max(Math.min(rect.y2,py0),rect.y1);
                        yb=Math.max(Math.min(rect.y2,py1),rect.y1);
						//System.out.println("points");
                        //for(x=xa;x<=xb;x+=echdx){
                        //    for(y=ya;y<=yb;y+=echdy){
                        // Correction de la boucle pour affichage des points quand
                        // la calibration est inverse.
                        for(x=Math.min(xa,xb);x<=Math.max(xa,xb);x+=echdx){
                            for(y=Math.min(ya,yb);y<=Math.max(ya,yb);y+=echdy){
                                xx=round(projection.projX(x,y)*referential.scalex);
                                yy=round(projection.projY(x,y)*referential.scaley);
                                gr.fillRect(xx,yy, 1,1) ;
                            }
                        }
                    }

                    if(mode==FILL)
                        {
                            int[] xPoints=new int[4];
                            int[] yPoints=new int[4];
                            xPoints[0]=round(x00);
                            yPoints[0]=round(y00);
                            xPoints[1]=round(x01);
                            yPoints[1]=round(y01);
                            xPoints[2]=round(x11);
                            yPoints[2]=round(y11);
                            xPoints[3]=round(x10);
                            yPoints[3]=round(y10);

                            gr.fillPolygon(xPoints,yPoints,4);
                        }
                    if(mode==LINEX)
                        {
                            double xa,xb,ya,yb;
                            double y;
                            xa=Math.max(Math.min(rect.x2,px0),rect.x1);
                            xb=Math.max(Math.min(rect.x2,px1),rect.x1);
                            ya=Math.max(Math.min(rect.y2,py0),rect.y1);
                            yb=Math.max(Math.min(rect.y2,py1),rect.y1);


                            for(y=ya;y<=yb;y+=echdy)
                                {
                                    gr.drawLine(round(projection.projX(xa,y)*referential.scalex),
                                                round(projection.projY(xa,y)*referential.scaley),
                                                round(projection.projX(xb,y)*referential.scalex),
                                                round(projection.projY(xb,y)*referential.scaley));
                                }
                        }
                    if(mode==LINEY)
                        {
                            double xa,xb,ya,yb;
                            double x;
                            xa=Math.max(Math.min(rect.x2,px0),rect.x1);
                            xb=Math.max(Math.min(rect.x2,px1),rect.x1);
                            ya=Math.max(Math.min(rect.y2,py0),rect.y1);
                            yb=Math.max(Math.min(rect.y2,py1),rect.y1);

                            for(x=xa;x<=xb;x+=echdx)
                                {
                                    gr.drawLine(round(projection.projX(x,ya)*referential.scalex),
                                                round(projection.projY(x,ya)*referential.scaley),
                                                round(projection.projX(x,yb)*referential.scalex),
                                                round(projection.projY(x,yb)*referential.scaley));
                                }
                        }


                }



        }

        
    };
