/**************************************************************************
*   vthierry@sophia.inria.fr, Copyright (C) 1996.  All rights reserved.   *
**************************************************************************/

package kartoMNT.duck.swing;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * This class encapsulates a Panel to input an int.
 */

public class JIntInputPanel extends JPanel {

    // Components of the Panel
    private JTextField textfield = null;
    private JScrollBar scrollbar = null;
    private int min = 0, max = 0, value = 0;
    Listener listener = new Listener();

   /** 
     * - Parameter:
     * @param name  Parameter name
     * - Optional parameters:
     * @param size  Panel textfield size in number of chars
     * @param value Initial value
     * @param min   Minimal value
     * @param max   Maximal value
     */
    public JIntInputPanel(String name, 
 			 int size, int value, int min, int max) 
        {build(name,size,value,min,max,DISPOSITION_NORMAL);
        }


    /** 
     * - Parameter:
     * @param name  Parameter name
     * - Optional parameters:
     * @param size  Panel textfield size in number of chars
     * @param value Initial value
     * @param min   Minimal value
     * @param max   Maximal value
     * @param max   Disposition: DISPOSITION_NORMAL or DISPOSITION_HORIZONTAL
     */
    public JIntInputPanel(String name, 
 			 int size, int value, int min, int max,int disposition)
         {build(name,size,value,min,max,disposition);}

    private void build(String name, 
 			 int size, int value, int min, int max,int disposition) 
    {

    if(disposition==DISPOSITION_HORIZONTAL)
        {setLayout(gridbag=new GridBagLayout());
         constraint = new GridBagConstraints();
         constraint.weightx=0;
         constraint.insets=new Insets(0,3,0,3);

        if (name != null)
            addc(new JLabel(name), constraint);
        textfield = (size == 0) ? new JTextField() : new JTextField(size);
        textfield.setMinimumSize(textfield.getPreferredSize());
        addc(textfield, constraint);

        if (min < max) 
            {
            this.min = min;
            this.max = max;
            
            scrollbar = new JScrollBar(JScrollBar.HORIZONTAL);
            scrollbar.setMinimum(min);
            scrollbar.setVisibleAmount(max/4);
            scrollbar.setMaximum(max + max/4);
            
            //constraint.weightx=1;
            constraint.weightx=1;
            constraint.fill=GridBagConstraints.HORIZONTAL;
            constraint.gridwidth = GridBagConstraints.REMAINDER;
            addc(scrollbar, constraint);
            }
        set(value);
        textfield.addActionListener(listener);
        if(scrollbar!=null) scrollbar.addAdjustmentListener(listener);
        }

    if(disposition==DISPOSITION_NORMAL)
        {
        setLayout(gridbag=new GridBagLayout());
        constraint = new GridBagConstraints();
        constraint.weightx=0;
        constraint.insets=new Insets(0,3,0,3);

        if (name != null)
            addc(new JLabel(name), constraint);
        textfield = (size == 0) ? new JTextField() : new JTextField(size);
        textfield.setMinimumSize(textfield.getPreferredSize());
        
        constraint.gridwidth = GridBagConstraints.REMAINDER ;
        constraint.anchor = GridBagConstraints.WEST;
        addc(textfield, constraint);

        if (min < max) 
            {
             this.min = min;
             this.max = max;
 
             scrollbar = new JScrollBar(JScrollBar.HORIZONTAL);
             scrollbar.setMinimum(min);
             scrollbar.setVisibleAmount(max/4);
             scrollbar.setMaximum(max + max/4);
             //scrollbar.addAdjustmentListener(listener);
             constraint.weightx=1;

             constraint.insets=new Insets(3,3,3,3);        
             constraint.fill=GridBagConstraints.HORIZONTAL;

             addc(scrollbar, constraint);
             }
    	set(value);
        textfield.addActionListener(listener);
        if(scrollbar!=null) scrollbar.addAdjustmentListener(listener);
        }

    /*setLayout(new BorderLayout());
 	if (name != null)
	    add(new JLabel(name), "West");
	textfield = (size == 0) ? new JTextField() : new JTextField(size);
	textfield.addActionListener(listener);
	add(textfield, "Center");
	if (min < max) {
	    this.min = min;
	    this.max = max;
	    scrollbar = new JScrollBar(JScrollBar.HORIZONTAL);
	    scrollbar.setMinimum(min);
	    scrollbar.setVisibleAmount(10);
	    scrollbar.setMaximum(max + 10);
	    scrollbar.addAdjustmentListener(listener);
	    add(scrollbar, "South");
	}
	set(value);*/
    }


    /*!*/ public JIntInputPanel(String name, int size, int value) {
	this(name, size, value, 0, 0);
    }
    /*!*/ public JIntInputPanel(String name, int value) {
	this(name, 0, value, 0, 0);
    }
    /*!*/ public JIntInputPanel(String name) {
	this(name, 0, 0, 0, 0);
    }
    /*!*/ public JIntInputPanel(String name, 
			       int value, int min, int max) {
	this(name, 0, value, min, max);
    }

    /** Runnable called on parameter input. 
     * - The Syntax <i>new JIntInputPanel(..) { public void run() { .. }};</i>
     allows to attach a call-back to this 
     * and the <i>get()</i> method to retrieve the parameter.
     */
    public void run() { }

    /** Get/Set the input value. */
    public int get() {
	return value;
    }
    public void set(int value) {
	this.value = value;
	if(scrollbar != null)
	    scrollbar.setValue(value);
	textfield.setText(String.valueOf(value));
    }

    private class Listener implements ActionListener, AdjustmentListener {

	/*** Implements the ScrollBar AdjustmentListener. */

	/*!*/ public final void adjustmentValueChanged(AdjustmentEvent event) {
	    value = scrollbar.getValue();
	    textfield.setText(String.valueOf(value));
	    run();
	}

	/*** Implements the JTextField ActionListener. */

	/*!*/ public final void actionPerformed(ActionEvent event) {
	    try {
		int value = Integer.valueOf(textfield.getText()).intValue();
		if ((min < max) &&  ((value < min) || (value > max)))
		    throw new NumberFormatException();
		JIntInputPanel.this.value = value;
		if(scrollbar != null)
		    scrollbar.setValue(value);
		run();
	    }  catch(NumberFormatException e) { 
		textfield.setText(String.valueOf(value));
	    }
	}
    }

private void addc(JComponent comp,GridBagConstraints c) 
    {gridbag.setConstraints(comp, c);
     add(comp);
    }
private GridBagConstraints constraint;
private GridBagLayout gridbag;       
public static final int DISPOSITION_HORIZONTAL=1;
public static final int DISPOSITION_NORMAL=2;

}


