/**
 * KartoMNT V2 <BR>
 *	Dialog du Visualisateur 3D de KartoMNT
 * @author DERHI Roland
 **/

package kartoMNT.visu3d;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import kartoMNT.*;

public class DialogVisu3DKartoMNT extends JDialog implements ItemListener, ActionListener{
    public static final int NO_TEX=0;
    public static final int TEX_COURANTE=1;
    public static final int TEX_CALIBREE=2;
    public static final int TEX_FICHIER=3;

	/** Lien vers le visualisateur 3D */
	private Visu3DKartoMNT demo;
	/** Radio Bouton de "Courant" */
	private JRadioButton rbCourant;
	/** Radio Bouton de "Fichier" (choix du MNT) */
	private JRadioButton rbFichier1;
	/** Radio Bouton de "Pas de Texture" (sur le MNT) */
	private JRadioButton rbNoTex;
    /** Radio Bouton de "Courante" (sur le MNT) */
	private JRadioButton rbTexCourante=null;
    /** Radio Bouton de "Calibre" (sur le MNT) */
	private JRadioButton rbCalibree;
	/** Radio Bouton de "Fichier" (choix de la texture du MNT) */
	private JRadioButton rbFichier2;
	/** Radio Bouton de "Pas de Texture" (de fond) */
	private JRadioButton rbNoTex3;
	/** Radio Bouton de "Fichier" (choix de la texture de fond) */
	private JRadioButton rbFichier3;
	/** Radio Bouton de "Automatique" */
	private JRadioButton rbParam1;
	/** Radio Bouton de "Manuel" */
	private JRadioButton rbParam2;
	/** JTextField de "Fichier" (choix du MNT) */
	private JTextField nomFichier1;
	/** JTextField de "Fichier" (choix de la texture du MNT) */
	private JTextField nomFichier2;
    /** JTextField de "Fichier" (choix de la texture calibre du MNT) */
	private JTextField nomFichier20;
	/** JTextField de "Fichier" (choix de la texture de fond) */
	private JTextField nomFichier3;
	/** JTextField de "gridScaleX" */
	private JTextField valGSX;
	/** JTextField de "gridScaleY" */
    private JTextField valGSY;
    /** JTextField de "zScale" */
	private JTextField valZS;
	/** JTextField de "zClip" */
	private JTextField valZC;
	/** JTextField de "LODDepth" */
	private JTextField valLD;
	/** JTextField de "LODLevel" */
	private JTextField valLL;
	/** panel "MNT" */
	private JPanel panelMNT;
	/** panel "Texture MNT" */
	private JPanel panelTextureMNT;
	/** panel "Texture de Fond" */
	private JPanel panelTextureBackground;
	/** panel "Parametres" */
	private JPanel panelParametre;
	/** bouton "..." (choix du MNT) */
	private JButton choixFichier1;
	/** bouton "..." (choix de la texture du MNT) */
	private JButton choixFichier2;
    /** bouton "..." (choix de la texture calibre du MNT) */
	private JButton choixFichier20;
	/** bouton "..." (choix de la texture de fond) */
	private JButton choixFichier3;
	/** bouton "OK" */
	private JButton bOK;
	/** bouton "Annuler" */
	private JButton bAN;
	/** bouton "Sauver" */
	private JButton bSA;
	/** JFileChooser de MNT */
	private JFileChooser fcMNT= new JFileChooser(".");
	/** JFileChooser d'images */
    private JFileChooser fcImage= new JFileChooser(".");
    /** JFileChooser de documents Karto */
    private JFileChooser fcKarto= new JFileChooser(".");
	/** Indique au Visualisateur si on a appuy sur OK */
	public boolean continu=false;
	/** Indique au Visualisateur si on a appuy sur Annuler */	
	public boolean termine=false;
	/** Le MNT courant */
    private MNT land=null;
    /** Le DocumentKarto courant */
    private DocumentKarto docKarto=null;
	
	/**
	 *	Constructeur
	 *	@param d le visualisateur 3D
	 *	@param land le MNT courant
	 *      @param docKarto le DocumentKarto courant
	 **/
	public DialogVisu3DKartoMNT(Visu3DKartoMNT d,MNT land,DocumentKarto docKarto){
		super((JFrame)null,ResourceManager.get("titleVisu"));
		demo=d;
		this.land=land;
		this.docKarto=docKarto;				

		// initilaisation des JFileChooser
		fcMNT.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fcImage.setFileSelectionMode(JFileChooser.FILES_ONLY);

		ExampleFileFilter filterImg = new ExampleFileFilter();
		filterImg.addExtension("jpg");
		filterImg.addExtension("gif");
		filterImg.setDescription(ResourceManager.get("images"));
		fcImage.addChoosableFileFilter(filterImg);
		
		//ExampleFileFilter filterMNT = new ExampleFileFilter();		
		//filterMNT.setDescription("Modele numerique de terrain");
		//fcMNT.addChoosableFileFilter(filterMNT);
		
		GestionnaireMntFormat.initFilter(fcMNT,MntFormat.IMPORT);

		ExampleFileFilter filterKarto = new ExampleFileFilter();
		filterKarto.addExtension("karto");		
		filterKarto.setDescription(ResourceManager.get("kartoDoc"));
		fcKarto.addChoosableFileFilter(filterKarto);

		// Construction de l'Interface
		Container contentPane=getContentPane();
		contentPane.setLayout(new BoxLayout(contentPane,BoxLayout.Y_AXIS));
		
		// Panel MNT
		panelMNT = new JPanel();
		panelMNT.setLayout(new GridLayout(3,1));

		TitledBorder title1 = BorderFactory.createTitledBorder(ResourceManager.get("mnt"));
		panelMNT.setBorder(title1);

		if (land !=null){
		    rbCourant = new JRadioButton(ResourceManager.get("curr"),Variables.mntCourant);
		    rbCourant.addItemListener(this);
		    panelMNT.add(rbCourant);
		    rbFichier1 = new JRadioButton(ResourceManager.get("file"),!Variables.mntCourant);
		}
		else{
		    rbFichier1 = new JRadioButton(ResourceManager.get("file"),true);
		}
		
		rbFichier1.addItemListener(this);
		panelMNT.add(rbFichier1);
		
		ButtonGroup rg1 = new ButtonGroup();
		if (land !=null)
		    rg1.add(rbCourant);
		rg1.add(rbFichier1);
		
		JPanel panelFichier1=new JPanel();
		panelFichier1.setLayout(new FlowLayout());
		JPanel panelVide =new JPanel();
		panelVide.setPreferredSize(new Dimension(20,10));
		nomFichier1 = new JTextField(20);
		nomFichier1.setText(Variables.mntURL);
		if (land !=null)
		    nomFichier1.setEditable(!Variables.mntCourant);
		else
		    nomFichier1.setEditable(true);
		choixFichier1 = new JButton("...");
		choixFichier1.addActionListener(this);
		panelFichier1.add(panelVide);
		panelFichier1.add(nomFichier1);
		panelFichier1.add(choixFichier1);
		panelMNT.add(panelFichier1);
		
		// Panel Texture MNT
		panelTextureMNT = new JPanel();
		panelTextureMNT.setLayout(new GridLayout(6,1));

		TitledBorder title2 = BorderFactory.createTitledBorder(ResourceManager.get("texMNT"));
		panelTextureMNT.setBorder(title2);

		rbNoTex = new JRadioButton(ResourceManager.get("notTex"),(Variables.texture==DialogVisu3DKartoMNT.NO_TEX));
		rbNoTex.addItemListener(this);
		panelTextureMNT.add(rbNoTex);
		if (docKarto!=null){
		    rbTexCourante = new JRadioButton(ResourceManager.get("curre"),(Variables.texture==DialogVisu3DKartoMNT.TEX_COURANTE));
		    rbTexCourante.addItemListener(this);
		    panelTextureMNT.add(rbTexCourante);
		}
		
		rbCalibree = new JRadioButton(ResourceManager.get("calib2"),(Variables.texture==DialogVisu3DKartoMNT.TEX_CALIBREE));
		rbCalibree.addItemListener(this);
		panelTextureMNT.add(rbCalibree);

		JPanel panelFichier20=new JPanel();
		panelFichier20.setLayout(new FlowLayout());
	
		nomFichier20 = new JTextField(20);
		nomFichier20.setText(Variables.textureCalibreeURL);
		nomFichier20.setEditable(Variables.texture==DialogVisu3DKartoMNT.TEX_CALIBREE);
		choixFichier20 = new JButton("...");
		choixFichier20.addActionListener(this);
		panelFichier20.add(panelVide);
		panelFichier20.add(nomFichier20);
		panelFichier20.add(choixFichier20);
		panelTextureMNT.add(panelFichier20);

		rbFichier2 = new JRadioButton(ResourceManager.get("file"),(Variables.texture==DialogVisu3DKartoMNT.TEX_FICHIER));
		rbFichier2.addItemListener(this);
		panelTextureMNT.add(rbFichier2);
		
		ButtonGroup rg2 = new ButtonGroup();
		rg2.add(rbNoTex);
		if (docKarto!=null) rg2.add(rbTexCourante);
		rg2.add(rbCalibree);
		rg2.add(rbFichier2);
		
		JPanel panelFichier2=new JPanel();
		panelFichier2.setLayout(new FlowLayout());
	
		nomFichier2 = new JTextField(20);
		nomFichier2.setText(Variables.textureURL);
		nomFichier2.setEditable(Variables.texture==3);
		choixFichier2 = new JButton("...");
		choixFichier2.addActionListener(this);
		panelFichier2.add(panelVide);
		panelFichier2.add(nomFichier2);
		panelFichier2.add(choixFichier2);
		panelTextureMNT.add(panelFichier2);
		
		
		// Panel Texture Background
		panelTextureBackground = new JPanel();
		panelTextureBackground.setLayout(new GridLayout(3,1));

		TitledBorder title3 = BorderFactory.createTitledBorder(ResourceManager.get("backTex"));
		panelTextureBackground.setBorder(title3);

		rbNoTex3 = new JRadioButton(ResourceManager.get("notTex"),!Variables.background);
		rbNoTex3.addItemListener(this);
		panelTextureBackground.add(rbNoTex3);
		rbFichier3 = new JRadioButton(ResourceManager.get("file"),Variables.background);
		rbFichier3.addItemListener(this);
		panelTextureBackground.add(rbFichier3);
		
		ButtonGroup rg3 = new ButtonGroup();
		rg3.add(rbNoTex3);
		rg3.add(rbFichier3);
		
		JPanel panelFichier3=new JPanel();
		panelFichier3.setLayout(new FlowLayout());
	
		nomFichier3 = new JTextField(20);
		nomFichier3.setText(Variables.backgroundURL);
		nomFichier3.setEditable(Variables.background);
		choixFichier3 = new JButton("...");
		choixFichier3.addActionListener(this);
		panelFichier3.add(panelVide);
		panelFichier3.add(nomFichier3);
		panelFichier3.add(choixFichier3);
		panelTextureBackground.add(panelFichier3);


			// Panel Parametre
		panelParametre = new JPanel();
		panelParametre.setLayout(new BoxLayout(panelParametre,BoxLayout.Y_AXIS));

		TitledBorder title4 = BorderFactory.createTitledBorder(ResourceManager.get("param"));
		panelParametre.setBorder(title4);

		JPanel panelF1=new JPanel(new FlowLayout(FlowLayout.LEFT));
		rbParam1 = new JRadioButton(ResourceManager.get("autoCons"),Variables.paramAuto);
		rbParam1.addItemListener(this);
		panelF1.add(rbParam1);
		panelParametre.add(panelF1);
		
		JPanel panelF2=new JPanel(new FlowLayout(FlowLayout.LEFT));
		rbParam2 = new JRadioButton(ResourceManager.get("manualNotCons"),!Variables.paramAuto);
		rbParam2.addItemListener(this);
		panelF2.add(rbParam2);
		panelParametre.add(panelF2);
		
		ButtonGroup rg4 = new ButtonGroup();
		rg4.add(rbParam1);
		rg4.add(rbParam2);
		
		
		// params
		JPanel panelParams=new JPanel();
		panelParams.setLayout(new GridLayout(3,2));

		// gridScaleX
		JPanel panelParam1=new JPanel();
		panelParam1.setLayout(new FlowLayout(FlowLayout.LEFT));
	
		JLabel lab1 = new JLabel("gridScaleX");
		lab1.setToolTipText(ResourceManager.get("gridScaleXHelp"));
		valGSX = new JTextField(5);
		valGSX.setText(Float.toString(Variables.gridScaleX));
		valGSX.setEditable(!Variables.paramAuto);				
		panelParam1.add(lab1);
		panelParam1.add(valGSX);
		

		// gridScaleY
		JPanel panelParam10=new JPanel();
		panelParam10.setLayout(new FlowLayout(FlowLayout.LEFT));
	
		JLabel lab10 = new JLabel("gridScaleY");
		lab10.setToolTipText(ResourceManager.get("gridScaleYHelp"));
		valGSY = new JTextField(5);
		valGSY.setText(Float.toString(Variables.gridScaleY));
		valGSY.setEditable(!Variables.paramAuto);				
		panelParam10.add(lab10);
		panelParam10.add(valGSY);
		

		// zScale
		JPanel panelParam2=new JPanel();
		panelParam2.setLayout(new FlowLayout(FlowLayout.LEFT));
	
		JLabel lab2 = new JLabel("zScale        ");
		lab2.setToolTipText(ResourceManager.get("zScaleHelp"));
		valZS = new JTextField(5);	
		valZS.setText( Double.toString(Variables.zScale));
		valZS.setEditable(!Variables.paramAuto);			
		panelParam2.add(lab2);
		panelParam2.add(valZS);
		
		
		// LODDepth
		JPanel panelParam3=new JPanel();
		panelParam3.setLayout(new FlowLayout(FlowLayout.LEFT));
	
		JLabel lab3 = new JLabel("LODDepth ");
		lab3.setToolTipText(ResourceManager.get("LODDepthHelp"));
		valLD = new JTextField(5);
		valLD.setText(Integer.toString(Variables.LODDepth));
		valLD.setEditable(!Variables.paramAuto);				
		panelParam3.add(lab3);
		panelParam3.add(valLD);
		
		
		// LODLevel
		JPanel panelParam4=new JPanel();
		panelParam4.setLayout(new FlowLayout(FlowLayout.LEFT));
	
		JLabel lab4 = new JLabel("LODLevel ");
		lab4.setToolTipText(ResourceManager.get("LODLevelHelp"));
		valLL = new JTextField(5);
		valLL.setText(Integer.toString(Variables.LODLevel));
		valLL.setEditable(!Variables.paramAuto);				
		panelParam4.add(lab4);
		panelParam4.add(valLL);
		
		
		// zClip
		JPanel panelParam5=new JPanel();
		panelParam5.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		JLabel lab5 = new JLabel("zClip          ");
		lab5.setToolTipText(ResourceManager.get("zClipHelp"));
		valZC = new JTextField(5);	
		valZC.setText( Float.toString(Variables.zClip));
		valZC.setEditable(!Variables.paramAuto);			
		panelParam5.add(lab5);
		panelParam5.add(valZC);
		

		
		
		panelParams.add(panelParam1);
		panelParams.add(panelParam4);
		panelParams.add(panelParam10);
		panelParams.add(panelParam3);
		panelParams.add(panelParam2);
		panelParams.add(panelParam5);
		
		panelParametre.add(panelParams);


		// Panel Bouton
		JPanel panelBouton = new JPanel();	
		panelBouton.setLayout(new BoxLayout(panelBouton,BoxLayout.X_AXIS));	
		bOK = new JButton(ResourceManager.get("OkKey"));
		bOK.addActionListener(this);
		bAN = new JButton(ResourceManager.get("CancelKey"));
		bAN.addActionListener(this);
		bSA = new JButton(ResourceManager.get("save"));
		bSA.addActionListener(this);
		panelBouton.add(bOK);
		panelBouton.add(bAN);
		panelBouton.add(bSA);
		
		
		// Ajout des Panels
		contentPane.add(panelMNT);
		contentPane.add(panelTextureMNT);
		contentPane.add(panelTextureBackground);
		contentPane.add(panelParametre);
		contentPane.add(panelBouton);


		// vrifie si l'utilisateur a cliqu sur la croix
		addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e)
		{
		    termine=true;
		}
		
	    });

		pack();
		setResizable(false);
		
		// positionnement du dialog
		Toolkit tk=Toolkit.getDefaultToolkit();
		Dimension d1=tk.getScreenSize();
		Dimension d2=getSize();
		setLocation((int)Math.max((d1.getWidth()-d2.getWidth())/2,0),(int)Math.max((d1.getHeight()-d2.getHeight())/2,0));
		
		setVisible(true);
	}	
	
	/**
	 *	Ecouteurs des Radio Boutons
	 *	@param ie vnement engendr lors de l'appui sur un Radio Bouton
	 **/
	public void itemStateChanged(ItemEvent ie){
		
		// Radio Bouton "Courant"
		if (ie.getSource() == rbCourant){
			nomFichier1.setEditable(false);			
		}
		// Radio Bouton "Fichier" (du MNT) 
		else if (ie.getSource() == rbFichier1){
			nomFichier1.setEditable(true);
			
		}
		// Radio Bouton "Pas de Texture" (sur le MNT)
		else if (ie.getSource() == rbNoTex){
			nomFichier2.setEditable(false);
			nomFichier20.setEditable(false);
		
		}
		// Radio Bouton "Courante" ( texture calibre du MNT) 
		else if (ie.getSource() == rbTexCourante){
		    nomFichier2.setEditable(false);
		    nomFichier20.setEditable(false);
		}
		// Radio Bouton "Calibre" ( texture du MNT) 
		else if (ie.getSource() == rbCalibree){
			nomFichier2.setEditable(false);
			nomFichier20.setEditable(true);

		}
		// Radio Bouton "Fichier" (de la texture du MNT) 
		else if (ie.getSource() == rbFichier2){
			nomFichier2.setEditable(true);
			nomFichier20.setEditable(false);

		}
		// Radio Bouton "Pas de Texture" (de fond)
		else if (ie.getSource() == rbNoTex3){
			nomFichier3.setEditable(false);

		}
		// Radio Bouton "Fichier" (de la texture de fond) 
		else if (ie.getSource() == rbFichier3){
			nomFichier3.setEditable(true);

		}
		// Radio Bouton "Automatique" 
		else if (ie.getSource() == rbParam1){
			valGSX.setEditable(false);
			valGSY.setEditable(false);
	 		valZS.setEditable(false);
			valZC.setEditable(false);
			valLD.setEditable(false);
		   valLL.setEditable(false);

		}
		// Radio Bouton "Manuel" 
		else if (ie.getSource() == rbParam2){
			valGSX.setEditable(true);
			valGSY.setEditable(true);
	 		valZS.setEditable(true);
			valZC.setEditable(true);
			valLD.setEditable(true);
		   valLL.setEditable(true);

		
		}
				
	}
	
	/**
	 * Mthode appel quand on appuie sur un bouton
	 *
	 * @param ae L'ActionEvent appel quand quelqu'un 
	 *           appuie sur un bouton
	 */
	public void actionPerformed(ActionEvent ae){
		JButton bouton = (JButton)ae.getSource();

		// bouton "..." (de MNT)
		if (bouton == choixFichier1){
			if (nomFichier1.isEditable()){
				
				int returnVal = fcMNT.showOpenDialog(null);
				String nomFichier=null;
			
				if (returnVal==JFileChooser.APPROVE_OPTION){
				    File file=fcMNT.getSelectedFile();
					// on vrifie si le fichier est accessible
				    if (file.exists()){
						if (!file.canRead()){
						    JOptionPane.showMessageDialog(null,ResourceManager.get("errorFileRead"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
						    return;
						}
						nomFichier=file.getPath();
					 }
				}
				if (nomFichier!=null){
				    nomFichier=nomFichier.replace('\\','/');
				
				    nomFichier1.setText("file:/"+nomFichier);
				}
			
			}
		}
		// bouton "..." (de texture du MNT)
		else if (bouton == choixFichier2){
			if (nomFichier2.isEditable()){
								
				int returnVal = fcImage.showOpenDialog(null);
				String nomFichier=null;
			
				if (returnVal==JFileChooser.APPROVE_OPTION){
				    File file=fcImage.getSelectedFile();
					// on vrifie si le fichier est accessible
				    if (file.exists()){
						if (!file.canRead()){
						    JOptionPane.showMessageDialog(null,ResourceManager.get("errorFileRead"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
						    return;
						}
						nomFichier=file.getPath();
					 }
				}
				if (nomFichier!=null){
				    nomFichier=nomFichier.replace('\\','/');				
				    nomFichier2.setText("file:"+((nomFichier.charAt(0)=='/')?"":"/")+nomFichier);
				}
			
			}

		}
		// bouton "..." (de texture calibre du MNT)
		else if (bouton == choixFichier20){
			if (nomFichier20.isEditable()){
								
				int returnVal = fcKarto.showOpenDialog(null);
				String nomFichier=null;
			
				if (returnVal==JFileChooser.APPROVE_OPTION){
				    File file=fcKarto.getSelectedFile();
					// on vrifie si le fichier est accessible
				    if (file.exists()){
						if (!file.canRead()){
						    JOptionPane.showMessageDialog(null,ResourceManager.get("errorFileRead"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
						    return;
						}
						nomFichier=file.getPath();
					 }
				}
				if (nomFichier!=null){
				    nomFichier=nomFichier.replace('\\','/');				
				    nomFichier20.setText(nomFichier);
				}
			
			}

		}
		// bouton "..." (de texture de fond)
		else if (bouton == choixFichier3){
			if (nomFichier3.isEditable()){
				
				int returnVal = fcImage.showOpenDialog(null);
				String nomFichier=null;
			
				if (returnVal==JFileChooser.APPROVE_OPTION){
				    File file=fcImage.getSelectedFile();
					// on vrifie si le fichier est accessible
				    if (file.exists()){
						if (!file.canRead()){
						    JOptionPane.showMessageDialog(null,ResourceManager.get("errorFileRead"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
						    return;
						}
						nomFichier=file.getPath();
					 }
				}
				if (nomFichier!=null){
				    nomFichier=nomFichier.replace('\\','/');				
				    nomFichier3.setText("file:"+((nomFichier.charAt(0)=='/')?"":"/")+nomFichier);
				}
			
			}

		}
		// bouton "OK"
		else if (bouton == bOK){
			boolean erreur=false;
			
			// on vrifie si le fichier spcifi pour le MNT est correct
			if (nomFichier1.isEditable()){
				File file = null;
				try{
					file = new File(nomFichier1.getText().substring(6));	
				}catch(Exception e){
					file = new File("");	
				}
				
				if (file.exists()){
					if (!file.canRead()){
						JOptionPane.showMessageDialog(null,ResourceManager.get("errorMNTRead"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
						return;
					}
				}
				else{
					JOptionPane.showMessageDialog(null,ResourceManager.get("errorMNTNotExist"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
					return;
				}
			}
			
			// on vrifie si le fichier spcifi pour la texture du MNT est correct
			if (nomFichier2.isEditable()){
				File file =  null;
				
				try{
				    
					file = new File(nomFichier2.getText().substring(5));	
				}catch(Exception e){
					file = new File("");	
				}
					
				if (file.exists()){
					if (!file.canRead()){
						JOptionPane.showMessageDialog(null,ResourceManager.get("errorMNTTexRead"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
						return;
					}
				}
				else{
					JOptionPane.showMessageDialog(null,ResourceManager.get("errorMNTTexNotExist"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
					return;
				}
			}

			// on vrifie si le fichier spcifi pour la texture du MNT est correct
			if (nomFichier20.isEditable()){
				File file =  null;
				
				try{
				    
					file = new File(nomFichier20.getText());	
				}catch(Exception e){
					file = new File("");	
				}
					
				if (file.exists()){
					if (!file.canRead()){
						JOptionPane.showMessageDialog(null,ResourceManager.get("errorMNTTexCalRead"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
						return;
					}
				}
				else{
					JOptionPane.showMessageDialog(null,ResourceManager.get("errorMNTTexCalNotExist"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
					return;
				}
			}
			
			// on vrifie si le fichier spcifi pour la texture de fond est correct
			if (nomFichier3.isEditable()){
				File file = null;
				try{
					file = new File(nomFichier3.getText().substring(5));	
				}catch(Exception e){
					file = new File("");	
				}
					
				if (file.exists()){
					if (!file.canRead()){
						JOptionPane.showMessageDialog(null,ResourceManager.get("errorMNTTexFondRead"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
						return;
					}
				}
				else{
					JOptionPane.showMessageDialog(null,ResourceManager.get("errorMNTTexFondNotExist"),ResourceManager.get("error"),JOptionPane.WARNING_MESSAGE);
					return;
				}
			}
			
			// on vrifie si le paramtre "gridScaleX" est correct
			try{
				Variables.gridScaleX = Float.parseFloat(valGSX.getText());
			}catch ( NumberFormatException e){
			    JOptionPane.showMessageDialog(null,ResourceManager.get("gridScaleXinc")
															,ResourceManager.get("error"),JOptionPane.ERROR_MESSAGE);
				erreur=true;
			}

			// on vrifie si le paramtre "gridScaleY" est correct
			try{
				Variables.gridScaleY = Float.parseFloat(valGSY.getText());
			}catch ( NumberFormatException e){
			    JOptionPane.showMessageDialog(null,ResourceManager.get("gridScaleYinc")
															,ResourceManager.get("error"),JOptionPane.ERROR_MESSAGE);
				erreur=true;
			}
			
			// on vrifie si le paramtre "zScale" est correct
			try{
				Variables.zScale = Float.parseFloat(valZS.getText());
			}catch ( NumberFormatException e){
			    JOptionPane.showMessageDialog(null,ResourceManager.get("zScaleinc")
															,ResourceManager.get("error"),JOptionPane.ERROR_MESSAGE);

				erreur=true;
			}
			
			// on vrifie si le paramtre "zClip" est correct
			try{
				Variables.zClip = Float.parseFloat(valZC.getText());
			}catch ( NumberFormatException e){
			    JOptionPane.showMessageDialog(null,ResourceManager.get("zClipinc")
															,ResourceManager.get("error"),JOptionPane.ERROR_MESSAGE);

				erreur=true;
			}

			// sauvegarde des nom de fichiers
			Variables.mntURL = nomFichier1.getText();
			Variables.textureURL= nomFichier2.getText();
			Variables.textureCalibreeURL= nomFichier20.getText();
			Variables.backgroundURL = nomFichier3.getText();
			
			// on vrifie si le paramtre "LODDepth" est correct
			try{
				Variables.LODDepth = Integer.parseInt(valLD.getText());
			}catch ( NumberFormatException e){
			    JOptionPane.showMessageDialog(null,ResourceManager.get("LODDepthinc")
															,ResourceManager.get("error"),JOptionPane.ERROR_MESSAGE);
				erreur=true;
			}
			
			// on vrifie si le paramtre "LODLevel" est correct
			try{
				Variables.LODLevel = Integer.parseInt(valLL.getText());
			}catch ( NumberFormatException e){
			    JOptionPane.showMessageDialog(null,ResourceManager.get("LODLevelinc")
															,ResourceManager.get("error"),JOptionPane.ERROR_MESSAGE);
				erreur=true;
			}

			// on sauvegarde les choix de l'utilsateur
			Variables.mntCourant=!nomFichier1.isEditable();
			if (rbNoTex.isSelected())
			    Variables.texture=DialogVisu3DKartoMNT.NO_TEX;
			else if ((rbTexCourante!=null)&&(rbTexCourante.isSelected()))
			    Variables.texture=DialogVisu3DKartoMNT.TEX_COURANTE;
			else if (rbCalibree.isSelected())
			    Variables.texture=DialogVisu3DKartoMNT.TEX_CALIBREE;
			else if (rbFichier2.isSelected())
			    Variables.texture=DialogVisu3DKartoMNT.TEX_FICHIER;

			Variables.background=nomFichier3.isEditable();
			Variables.paramAuto=!valGSX.isEditable();
					
			// s'il n'y a pas eu d'erreur, on peut continuer,
			if (!erreur)
				continu=true;
		}
		// bouton "Annuler"
		else if (bouton == bAN){
			termine=true;

		}
		// bouton "Sauver"
		else if (bouton == bSA){
			// on sauvegarde l'ensemble des variables
			Variables.setVariable("gridScaleX",valGSX.getText());
			Variables.setVariable("gridScaleY",valGSY.getText());
			Variables.setVariable("zScale",valZS.getText());
			Variables.setVariable("zClip",valZC.getText());
			Variables.setVariable("mntURL",nomFichier1.getText());
			Variables.setVariable("textureURL",nomFichier2.getText());
			Variables.setVariable("textureCalibreeURL",nomFichier20.getText());
			Variables.setVariable("backgroundURL",nomFichier3.getText());
			Variables.setVariable("LODDepth",valLD.getText());
			Variables.setVariable("LODLevel",valLL.getText());

			Variables.setVariable("mntCourant",(new Boolean(!nomFichier1.isEditable())).toString());
			//Variables.setVariable("texture",(new Boolean(nomFichier2.isEditable())).toString());
			if (rbNoTex.isSelected())
			    Variables.setVariable("texture",Integer.toString(DialogVisu3DKartoMNT.NO_TEX));
			else if ((rbTexCourante!=null)&&(rbTexCourante.isSelected()))
			    Variables.setVariable("texture",Integer.toString(DialogVisu3DKartoMNT.TEX_COURANTE));
			else if (rbCalibree.isSelected())
			    Variables.setVariable("texture",Integer.toString(DialogVisu3DKartoMNT.TEX_CALIBREE));
			else if (rbFichier2.isSelected())
			    Variables.setVariable("texture",Integer.toString(DialogVisu3DKartoMNT.TEX_FICHIER));
			
			Variables.setVariable("background",(new Boolean(nomFichier3.isEditable())).toString());
			Variables.setVariable("paramAuto",(new Boolean(!valGSX.isEditable())).toString());

		}
		else{
		}

	}


}
