package kartoMNT.visu3d;

import java.io.*;
import com.sun.j3d.utils.image.*;
import java.applet.Applet;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import java.awt.event.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;
import java.net.*;

import com.sun.j3d.utils.geometry.*;

import kartoMNT.*;


/**
 * <P>
 * KartoMNT V2 <BR>
 *	Le Terrain 3D reprsenant le MNT <BR>
 * <BR>
 * Ce travail est inspir de celui de Harri Salonen (48031W) <BR>
 * de l'Helsinki University of Technology<BR>
 * <BR>
 * Ralis sous la direction de LINGRAND Diane et FERNANDES Gilbert <BR>
 * Ecole Suprieure en Sciences Informatiques de Sophia-Antipolis (FRANCE)
 * </P>
 *	
 * @author DERHI Roland
 */

public class LandScape {
   
   /** texture du Terrain 3D */
   private Texture earth;
	/** Valeurs du MNT */
	private float[][] heightValues;
    /** Scaling of grids dep */
    private float debGridScaleX = 0.07f;
    /** Scaling of grids dep */
    private float debGridScaleY = 0.07f;
	/** Scaling of grids X */
	private float gridScaleX = 0.07f;
    /** Scaling of grids Y */
	private float gridScaleY = 0.07f;
	/** Scaling of height */
	private float zScale = 0.0022f;
	/** Appearance selector */
	private boolean wireFrame = true;
	/** Image X component size */
	private int xSize;
	/** Image Y component size */
	private int ySize;
	/** Number is LOD models */
	private int LODDepth ;
	/** LOD switching distance */
    private float LODDistance = 500.0f;
	/** LOD subdivision level */
	private int LODLevel ;
	/** Texture for landscape */
	private Texture tex = null;
	/** Stretch grid a little bit in multilevel, dirty hack, not used */
	private float floatingCorrectionFactor = 0.4f;
	/** Ambient color */
	private Color3f ambientColor = new Color3f(0.4f, 0.4f, 0.4f);
	/** Diffuse color */
	private Color3f diffuseColor = new Color3f(1.0f, 1.0f, 1.0f);
	/** Table contenant l'ensemble des Shape3D composant le Terrain 3D */
	private ArrayList arrayShape=new ArrayList();
	/** Apparence de fil de fer */
	private Appearance wireFrameAppearance = createWireFrameAppearance();
	/** Table des normales aux Triangles composant le Terrain 3D */
   private Vector3f normals[];
   /** Table contenant l'ensemble des DistanceLOD du Terrain 3D */
	private ArrayList arrayDL = new ArrayList();
	/** ProgressMonitor de chargement du Terrain 3D */
	private ProgressMonitor progMon;
	/** Indice de chargement du Terrain 3D */
	private int chargement=0;
	/** Le MNT que reprsente le Terrain 3D */
    private MNT mnt;
    
    /**
     *	Constructeur
	 * @param MNT Le MNT que reprsente le Terrain 3D	 
	 */
   public LandScape(MNT mnt) {
       
       this.mnt=mnt;
       // mme si cela peut sembler bizzare, cela n'est pas une erreur
       // mais plutt un "patch" permettant de corriger une erreur et 
       // viter ainsi de retoucher des dizaines de lignes de code...
		this.xSize = mnt.getYMax();
		this.ySize = mnt.getXMax();

		// cration de la table des valeurs du MNT
		heightValues = new float[ySize][xSize];
		
		for (int j = 0; j < ySize; j++) {
		    for (int i = 0; i < xSize; i++) {
			heightValues[j][i] = mnt.getValue(j,xSize-i-1);
		    }
		}
		
		// prcalcul des normales aux Triangles
		precalculNormal();
		
		
		if (Variables.paramAuto){
			// calcul automatique des valeurs optimales
			LODDepth = (int)Math.ceil(((int)Math.floor(Math.log(Math.min(xSize, ySize))/Math.log(2)))/2.0);
			LODLevel = (int)Math.floor(((int)Math.floor(Math.log(Math.min(xSize, ySize))/Math.log(2)))/2.0);
		}
		else{
			// entre manuelle des valeurs par l'utilisateur
			LODDepth = Variables.LODDepth;
			LODLevel = Variables.LODLevel;
		}
		System.out.println("LODDepth : "+LODDepth+" et LODLevel : "+ LODLevel);
		
		// Initialisation du ProgressMonitor
		progMon= new ProgressMonitor(null,ResourceManager.get("landBuild"),ResourceManager.get("pleaseWait"),0,(int)((xSize-1)*(ySize-1)*((1-Math.pow(0.25,LODDepth+1))/0.75)));
		progMon.setProgress(chargement);
		progMon.setMillisToPopup(0);

   }   	
	
	/**
	 * Read configuration file
	 * (utile pour test, plus utilis)
	 * @param config name of the config file
	 */
	public void readConfig(String config) {
		BufferedReader cfg = null;
		String line;
		try {
			cfg = new BufferedReader(new FileReader(config));
			do {
				line = cfg.readLine();
			} while (line.indexOf("//") != -1);
			LODDepth = Integer.parseInt(line);
			do {
				line = cfg.readLine();
			} while (line.indexOf("//") != -1);
			LODLevel = Integer.parseInt(line);			
			do {
				line = cfg.readLine();
			} while (line.indexOf("//") != -1);
			LODDistance = Float.parseFloat(line);
			cfg.close();
		} catch (Exception e) {
			System.out.println("Config read failed: "+e+"!");
		} 
		System.out.print("LOD depth: "+LODDepth);
		System.out.print(" | LOD level: "+LODLevel);
		System.out.print(" | LOD distance: "+LODDistance);
	}
	/**
	 * Computes the normal vector of three vertice.
	 * Red book: normal = [v1 - v2] x [v2 - v3]
	 * @param v1 1st vertex
	 * @param v2 2nd vertex 
	 * @param v3 3rd vertex
	 * @return created normal
	 */
	private Vector3f findNormal(Point3f v1, Point3f v2, Point3f v3) {
		Vector3f normal = new Vector3f();
		v1.sub(v2);
		v2.sub(v3);
		normal.cross(new Vector3f(v1),new Vector3f(v2));
		return normal;
	}
	
	
	/**
	 * Set the scaling X factor for grids.
	 * The scaling is applied, when new shape is formed.
	 * <pre>
	 * -- X --
	 * |     |		
	 * Y     Y     
	 * |     |     
	 * -- X --
	 * </pre>
	 * @param newScaleX scale to apply in X
	 */
	public void setGridScaleX(float newScaleX) {
	    Variables.gridScaleX=newScaleX;			
	    debGridScaleX = newScaleX;
		gridScaleX = newScaleX;		
	}

	/**
	 * Set the scaling Y factor for grids.
	 * The scaling is applied, when new shape is formed.
	 * <pre>
	 * -- X --
	 * |     |		
	 * Y     Y     
	 * |     |     
	 * -- X --
	 * </pre>
	 * @param newScaleY scale to apply in Y
	 */
    public void setGridScaleY(float newScaleY) {
	Variables.gridScaleY=newScaleY;
	    debGridScaleY = newScaleY;
		gridScaleY = newScaleY;		
	}
	
	/** 
	 * Accesseur du gridScaleX de dpart 
	 * @return gridScaleX
	 */
	public float getGridScaleX(){
		return debGridScaleX;
	}

	/** 
	 * Accesseur du gridScaleY de dpart 
	 * @return gridScaleY
	 */
    public float getGridScaleY(){
		return debGridScaleY;
	}

	/** 	
	 * Accesseur du zScale 
	 * @return zScale
	 */
    public float getZScale(){
	return zScale;
    }

	/** 
	 *	Accesseur du MNT
	 *	@return le MNT
	 */
    public MNT getMNT(){
	return mnt;
    }
	
	/**
	 * Set the scaling factor for grids.
	 * This scales the heights of the mountains.
	 * @param newScale scale to apply
	 */
	public void setHeightScale(float newScale) {
		zScale = newScale;
	}	
	
	

	/**
	 * Select material model.
	 */
	public void selectMaterial() {
		wireFrame = false;
	}

	/**
	 * Select wireframe model.
	 */
	public void selectWireframe() {
		wireFrame = true;
	}
	
	/**
	 *	Explode the terrain. 
	 * You must reconstruct model after this.
	 * @param x origin of explosion
	 * @param y origin of explosion
	 * @param power power of explosion
	 */
	public void explode(int x, int y, int power) {
		
		heightValues[y][x] -= 2*power;
		
		if (x == 0) 
			x = heightValues.length - 1;
		else x--;
		
		heightValues[y][x] -= power;
				
		if (y == 0) 
			y = heightValues[0].length - 1;
		else y--;
		
		heightValues[y][x] -= power;

		if (x == heightValues.length - 1) 
			x = 0;
		else x++;
		
		heightValues[y][x] -= power;

		if (x == heightValues.length - 1) 
			x = 0;
		else x++;

		heightValues[y][x] -= power;

		if (y == heightValues[0].length - 1) 
			y = 0;
		else y++;
	
		heightValues[y][x] -= power;
		
		if (y == heightValues[0].length - 1) 
			y = 0;
		else y++;
	
		heightValues[y][x] -= power;
		
		if (x == 0) 
			x = heightValues.length - 1;
		else x--;

		heightValues[y][x] -= power;
		
		if (x == 0) 
			x = heightValues.length - 1;
		else x--;
		
		heightValues[y][x] -= power;	
	}
	
	/** 
	 * Set texture to whole landscape. 
	 * Texture is mapped to each square of land separately.
	 * Note! Material mode must be selected!
	 * @param tex texture to apply
	 */
	public void setTexture(Texture tex) {
		this.tex = tex;	    
	}

	/** 
	 * Set ambient color to whole landscape. 
	 * Note! Material mode must be selected!
	 * @param r red value
	 * @param g green value
	 * @param b blue value
	 */
	public void setAmbientColor(float r, float g, float b) {
		ambientColor = new Color3f(r,g,b);
	}
	
	/** 
	 * Set diffuse color to whole landscape. 
	 * Note! Material mode must be selected!
	 * @param r red value
	 * @param g green value
	 * @param b blue value
	 */
	public void setDiffuseColor(float r, float g, float b) {
		diffuseColor = new Color3f(r,g,b);
	}
	
	
	
	/** 
	 * Set LOD values.
	 * @param LODDepth is the number of lower res models
	 * @param LODDistance is the distance before swithing to a lower res
	 * @param LODLevel is the subdivision level
	 */
	public void setLOD(int LODDepth, float LODDistance, int LODLevel) {
		if (LODDepth >= 0 && LODDepth < 30) 
			this.LODDepth = LODDepth;
		if (LODDistance > 0.0f) 
			this.LODDistance = LODDistance;
		if (LODLevel >= 0 && LODLevel < 30) 
			this.LODLevel = LODLevel;						
	}
	
	/**
	 * Calculate the average of six normals surrounding
	 * a single point.
	 * @param v1, 1st normal
	 * @param v2, 2nd normal
	 * @param v3, 3rd normal
	 * @param v4, 4th normal
	 * @param v5, 5th normal
	 * @param v6, 6th normal	 
	 * @return averaged normal
	 */
	private Vector3f normalAvg(Vector3f v1, Vector3f v2, Vector3f v3,
										Vector3f v4, Vector3f v5, Vector3f v6) {
		
		Vector3f normal = new Vector3f();									
		normal.add(v1);
		normal.add(v2);
		normal.add(v3);
		normal.add(v4);
		normal.add(v5);
		normal.add(v6);
		normal.normalize();
		return normal;
	}
	 
	
	
	/**
    * Create appearance of material surface.
	 * @return created appearance
	 */
	private Appearance createMaterialAppearance(){

		Appearance materialAppear = new Appearance();
      PolygonAttributes polyAttrib = new PolygonAttributes();
      polyAttrib.setCullFace(PolygonAttributes.CULL_NONE);
      materialAppear.setPolygonAttributes(polyAttrib);

      Material material = new Material();
      material.setDiffuseColor(diffuseColor);
      material.setAmbientColor(ambientColor);		
	material.setSpecularColor(0.0f,0.0f,0.0f);
      materialAppear.setMaterial(material);
		
		if (tex != null) {
			materialAppear.setTexture(tex);
    		TextureAttributes texAttr = new TextureAttributes();
    		texAttr.setTextureMode(TextureAttributes.MODULATE);
    		materialAppear.setTextureAttributes(texAttr);
      }			
      return materialAppear;
   }

	/**
    	 * Create appearance of no surface (wireframe).
	 * @return created appearance
	 */
	private Appearance createWireFrameAppearance(){
      Appearance materialAppear = new Appearance();
      PolygonAttributes polyAttrib = new PolygonAttributes();
      polyAttrib.setPolygonMode(PolygonAttributes.POLYGON_LINE);
 	polyAttrib.setCullFace(PolygonAttributes.CULL_NONE);
      materialAppear.setPolygonAttributes(polyAttrib);
      ColoringAttributes redColoring = new ColoringAttributes();
      redColoring.setColor(1.0f, 0.0f, 0.0f);
      materialAppear.setColoringAttributes(redColoring);

      return materialAppear;
   }	

	/** 
	 * Creates a multiplied model, LOD should
	 * help to simplify the heavy load..
	 * (non utilis)
	 * @param gridSize , number of blocks in each side, eg. 3 creates 3x3 grid
	 * @return newly created BranchGroup
	 */
	public BranchGroup multiModel(int gridSize) {
		if (gridSize % 2 == 0 || gridSize < 3 || gridSize > 30)
			return null;
		
		BranchGroup B = new BranchGroup();
		for (int i = (1-gridSize)/2; i <= (gridSize-1)/2; i++)
			for (int j = (1-gridSize)/2; j <= (gridSize-1)/2; j++) {
			   Transform3D objTransf = new Transform3D();
			   objTransf.set(new Vector3f(i*gridScaleX*(xSize-1),0.0f,j*gridScaleY*(ySize-1)));
				TransformGroup T = new TransformGroup(objTransf);
				//T.addChild(getShape());
				T.addChild(LODModel());
				B.addChild(T);
			}
		B.compile();
		return B;
	}

	/**
	 * Return simplified values for the heightfield array
	 * @param res exponential reduction factor
	 * @param oldValues values to reduce
	 * @return array with (dimensions >> factor)
	 */
    private float[][] newValues(int res, float[][] oldValues) {
		// Get every n:th value (n =(2^res))
	int w = (oldValues[0].length >> res);
		int h = (oldValues.length >> res);

		float[][] values = new float[h][w];
		int skip = 1 << res;
	   for (int j = 0; j < h-1; j++) {
	   	for (int i = 0; i < w-1; i++) {
				values[j][i] = oldValues[j*skip][i*skip];
	      }
	   }
		// Create mathing sides
		for (int i = 0; i < h-1; i++) {
			values[i][w-1] = oldValues[i*skip][oldValues[0].length-1];
		}
		for (int i = 0; i < w-1; i++) {
			values[h-1][i] = oldValues[oldValues.length-1][i*skip];
		}

		values[h-1][w-1] = oldValues[oldValues.length-1][oldValues[0].length-1];
		
		return values;
	}
	

	/** 
	 * Create LOD of the landscape defined by
	 * values grid.
	 * @param values values to create LOD from
	 * @param x indice en X de values par rapport  heightValues
	 * @param y indice en Y de values par rapport  heightValues
	 * @return new LOD TransformGroup
	 */
	    private TransformGroup createLOD(float[][] values,int x,int y) {

		if (LODDepth < 2) {
			TransformGroup T = new TransformGroup();
			Appearance appear;		
			if (wireFrame)	
				appear = createWireFrameAppearance();
			else 
				appear = createMaterialAppearance();	
			Shape3D shtemp;							
	      T.addChild(shtemp=new Shape3D(convertToArray(values,x,y), appear));
	      shtemp.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
	      shtemp.setAppearanceOverrideEnable(true);
	      arrayShape.add(shtemp);
			return T;
		}
		float[] distances = new float[LODDepth-1];
		for (int i = 1; i <LODDepth; i++)
			distances[i-1] = i * LODDistance;
			
		BoundingSphere bounds = new BoundingSphere(new Point3d(),10000);

		// create target TransformGroup with Capabilities
		TransformGroup T = new TransformGroup();
      T.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

      // create DistanceLOD target object
      Switch targetSwitch = new Switch();
      targetSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);
	
		// create low-res models
		float oldGridScaleX = gridScaleX;
		float oldGridScaleY = gridScaleY;
		for (int i = 0; i <LODDepth; i++) {
			Appearance appear;		
			if (wireFrame)	
				appear = createWireFrameAppearance();
			else 
				appear = createMaterialAppearance();								

			float [][] newGrid = newValues(i,values);
			gridScaleX = oldGridScaleX * ((float)(values[0].length-1))/((float)(newGrid[0].length-1));
			gridScaleY = oldGridScaleY * ((float)(values.length-1))/((float)(newGrid.length-1));
			Shape3D shtemp;		
	      targetSwitch.addChild(shtemp=new Shape3D(convertToArray(newGrid,x,y,i), appear));
	      shtemp.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
	      shtemp.setAppearanceOverrideEnable(true);
			arrayShape.add(shtemp);
		}
		gridScaleX = oldGridScaleX;
		gridScaleY = oldGridScaleY;
		
      // create DistanceLOD object
		Point3f p;
      
      DistanceLOD dLOD = new DistanceLOD(distances, p=new Point3f(values[0].length/2.0f*debGridScaleX,values[values.length/2][values[0].length/2]* zScale,values.length/2.0f*debGridScaleY));
		arrayDL.add(dLOD);


      dLOD.addSwitch(targetSwitch);
      dLOD.setSchedulingBounds(bounds);
		T.addChild(targetSwitch);
		T.addChild(dLOD);
		
		return T;
	}
	

    /** 
	 * Divide landscape to smaller blocks
	 * @param values height values
	 * @param level level of subdivision
	 */
    private BranchGroup LODSubdivide(float[][] values, int level) {
	return LODSubdivide(values,level,0,0);
    }
	
	/** 
	 * Divide landscape to smaller blocks
	 * @param values height values
	 * @param level level of subdivision
	 * @param x indice en X de values par rapport  heightValues
	 * @param y indice en Y de values par rapport  heightValues
	 */
    private BranchGroup LODSubdivide(float[][] values, int level,int x,int y) {
		
		BranchGroup B = new BranchGroup();
		if (level == 0)
			B.addChild(createLOD(values,x,y));
		else {
		    int w = values[0].length;
		    int w2 = values[0].length/2;
		    int h = values.length;
		    int h2 = values.length/2;
			System.out.println(w+":"+w2+" x "+h+":"+h2);

			boolean estPairW=(w%2==0);
			boolean estPairH=(h%2==0);

			// cration des 4 subdivisions du Terrain 3D
			float[][] values1 = new float[h2+1][w2+1];
			float[][] values2 = new float[h2+1][(estPairW)?w2:w2+1];
			float[][] values3 = new float[(estPairH)?h2:h2+1][w2+1];
			float[][] values4 = new float[(estPairH)?h2:h2+1][(estPairW)?w2:w2+1];
			
			// on remplit les valeurs des subdivisions
			for (int i = 0; i <= h2; i++)
				for (int j = 0; j <= w2; j++) 
					values1[i][j] = values[i][j];	
		

			for (int i = 0; i <= h2; i++)
			    for (int j = w2; j < ((estPairW)?w-1:w); j++) {
				if (estPairW){
				    values2[i][j-w2+1] = values[i][j+1];
				}
				else{
				    values2[i][j-w2] = values[i][j];		    
				}
			    }

				if (estPairW)
			    	for (int i = 0; i <= h2; i++) {			    
					values2[i][0] = values[i][w2];
			    }
			
			

	
				for (int i = h2; i < ((estPairH)?h-1:h); i++)
				    for (int j = 0; j <= w2; j++) {
					if (estPairH)				     
					    values3[i-h2+1][j] = values[i+1][j];
					else 
					    values3[i-h2][j] = values[i][j]; 
				    }
				
	
				if (estPairH)
				    for (int j = 0; j <= w2; j++) {			    
					values3[0][j] = values[h2][j];
				   	}
	

			for (int i = h2; i < ((estPairH)?h-1:h); i++)
			    for (int j = w2; j < ((estPairW)?w-1:w); j++) {
				if (estPairH){				    
				    if (estPairW){
					values4[i-h2+1][j-w2+1] = values[i+1][j+1]; 
				    }
				    else{
					values4[i-h2+1][j-w2] = values[i+1][j]; 
				    }

				}
				else{
				    if (estPairW){
					values4[i-h2][j-w2+1] = values[i][j+1];
				    }
				    else{
					values4[i-h2][j-w2] = values[i][j];	
				    }
				}
			    }

			if (estPairH)
			    for (int j = w2; j < w; j++) {   				
				values4[0][j-w2] = values[h2][j];
			    }

			if (estPairW)
			    for (int i = h2; i < h; i++) {			    
				values4[i-h2][0] = values[i][w2];
			    }


				
			float steph = gridScaleY  * h2;
			float stepw = gridScaleX  * w2;
			
			
			TransformGroup T1 = new TransformGroup();
			T1.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE); 			
			
			Transform3D T3D2 = new Transform3D();
			T3D2.set(new Vector3f(stepw,0.0f,0.0f));			
			TransformGroup T2 = new TransformGroup(T3D2);
			T2.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE); 			
			
			Transform3D T3D3 = new Transform3D();
			T3D3.set(new Vector3f(0.0f,0.0f,steph));
			TransformGroup T3 = new TransformGroup(T3D3);
			T3.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE); 			

			Transform3D T3D4 = new Transform3D();
			T3D4.set(new Vector3f(stepw,0.0f,steph));
			TransformGroup T4 = new TransformGroup(T3D4);
			T4.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE); 			
			
			// on ritre sur chaque subdivision
			T1.addChild(LODSubdivide(values1, level-1,x,y));
			B.addChild(T1);
			T2.addChild(LODSubdivide(values2, level-1,x+w2,y));
			B.addChild(T2);
			T3.addChild(LODSubdivide(values3, level-1,x,y+h2));
			B.addChild(T3);
			T4.addChild(LODSubdivide(values4, level-1,x+w2,y+h2));
			B.addChild(T4);
		}
		B.compile();
		return B;
	}
	
	/** 
	 * Create model with Level Of Detail.
	 * The greater the distance, the simpler the
	 * model.
	 * @return BranchGroup representing the model
	 */
	public BranchGroup LODModel() {
		BranchGroup B = new BranchGroup();
		B.addChild(LODSubdivide(heightValues,LODLevel));		
		progMon.close();
		return B;
	}
	
	/** 
	 * Creates a landscape using triangle arrays.
	 * @return created object
	 */
	public Shape3D getShape() {
		Appearance appear;		
		if (wireFrame)	
			appear = createWireFrameAppearance();
		else 
			appear = createMaterialAppearance();			
		return new Shape3D(convertToArray(heightValues), appear);
	}
	
	/**
	 *	Accesseur de wireFrame
	 * (Permet de connatre le mode d'affichage
	 * @return wireframe
	 **/
	public boolean getWireframe(){
		return wireFrame;
	}
	
	/**
	 * Mthode de prcalcul des normales
	 */
	public void precalculNormal(){
		normals = new Vector3f[(xSize-1)*(ySize-1)*2+1];	
		int normalNr = 0;
		Point3f v1 = null, v2 = null, v3 = null;
		float coord[] = new float[3];
		for (int j = 0; j < ySize-1; j++) {
	   		for (int i = 0; i < xSize-1; i++) {
			   // (1)
				coord[0] = i * gridScaleX;
				coord[2] = (j+1) * gridScaleY;
				coord[1] = heightValues[j+1][i] * zScale;
				v1 = new Point3f(coord);
				// (2)
				coord[0] = (i+1) * gridScaleX;
				coord[2] = j * gridScaleY;
				coord[1] = heightValues[j][i+1] * zScale;
				v2 = new Point3f(coord);
				// (3)
				coord[0] = i * gridScaleX;
				coord[2] = j * gridScaleY;
				coord[1] = heightValues[j][i] * zScale;							
				v3 = new Point3f(coord);
				// Triangle ready - calculate normal				
				normals[normalNr++] = findNormal(v1,v2,v3);
				// (4)
				coord[0] = i * gridScaleX;
				coord[2] = (j+1) * gridScaleY;
				coord[1] = heightValues[j+1][i] * zScale;
				v1 = new Point3f(coord);
				// (5)
				coord[0] = (i+1) * gridScaleX;
				coord[2] = (j+1) * gridScaleY;
				coord[1] = heightValues[j+1][i+1] * zScale;								
				v2 = new Point3f(coord);				
				// (6)
				coord[0] = (i+1) * gridScaleX;
				coord[2] = j * gridScaleY;
				coord[1] = heightValues[j][i+1] * zScale;
				v3 = new Point3f(coord);				
				// Triangle ready - calculate normal				
				normals[normalNr++] = findNormal(v1,v2,v3);				

			}
		}
		// Set null normal
		normals[normalNr] = new Vector3f();
		
		
		
	}
	

	/**
	 * Convert heightmap values to triangle array,
	 * with textures, normals and everything..
	 * @param values heightmap values
	 * @return created array
	 */
    private TriangleArray convertToArray(float[][] values) {
		return convertToArray(values,0,0,0);
    }

	/**
	 * Convert heightmap values to triangle array,
	 * with textures, normals and everything..
	 * @param values heightmap values
 	 * @param x indice en X de values par rapport  heightValues
	 * @param y indice en Y de values par rapport  heightValues
	 * @return created array
	 */
    private TriangleArray convertToArray(float[][] values,int x,int y) {
		return convertToArray(values,x,y,0);
    }        

	/**
	 * Convert heightmap values to triangle array,
	 * with textures, normals and everything..
	 * @param values heightmap values
 	 * @param x indice en X de values par rapport  heightValues
	 * @param y indice en Y de values par rapport  heightValues
	 * @param depth nombre de subdivision qu'a dj connue values 
	 * @return created array
	 */
    private TriangleArray convertToArray(float[][] values,int x,int y,int depth) {

		if (values == null) return null;
		
		int w = values[0].length-1;
		int h = values.length-1;
		// multiflicateur  appliquer aux coordonnes de texture
		int multDepth = (int)Math.pow(2,depth);
		// multiplicateurs  appliquer aux coordonnes en X et Y
		float multScaleX = gridScaleX/debGridScaleX;
		float multScaleY = gridScaleY/debGridScaleY;


		 
		System.out.print("Creating grid: "+w+"x"+h+" ("+w*h*2+" polygons)..");
		TriangleArray T = new TriangleArray(w * h * 6, 	GeometryArray.COORDINATES | 
																		GeometryArray.TEXTURE_COORDINATE_2 |
																		GeometryArray.NORMALS);
		
		double coord[] = new double[3];
		float texCoord[]= new float[2];

		int coordNr = 0, normalNr = (xSize-1)*(ySize-1)*2;

	   for (int j = 0; j < h; j++) {
	   	for (int i = 0; i < w; i++) {
				progMon.setProgress(++chargement);
				
				/*         
						3 -- 2         6
						|  /         / |    
						| /         /  |
						1         4 -- 5
				*/

				// (1)
				coord[0] = i * gridScaleX;
				coord[2] = (j+1) * gridScaleY;
				coord[1] = values[j+1][i] * zScale;
				T.setCoordinate(coordNr, coord);

				texCoord[0] = (y+(j+1)*multScaleY)/((float)ySize);
				texCoord[1] = (x+i*multScaleX)/((float)xSize);
				T.setTextureCoordinate(0, coordNr++, texCoord);

				// (2)
				coord[0] = (i+1) * gridScaleX;
				coord[2] = j * gridScaleY;
				coord[1] = values[j][i+1] * zScale;
				T.setCoordinate(coordNr, coord);

				texCoord[0] = (y+j*multScaleY)/((float)ySize);
				texCoord[1] = (x+(i+1)*multScaleX)/((float)xSize);
				T.setTextureCoordinate(0, coordNr++, texCoord);

				// (3)
				coord[0] = i * gridScaleX;
				coord[2] = j * gridScaleY;
				coord[1] = values[j][i] * zScale;							
				T.setCoordinate(coordNr, coord);

				texCoord[0] = (y+j*multScaleY)/((float)ySize);
				texCoord[1] = (x+i*multScaleX)/((float)xSize);
				T.setTextureCoordinate(0, coordNr++, texCoord);

				// (4)
				coord[0] = i * gridScaleX;
				coord[2] = (j+1) * gridScaleY;
				coord[1] = values[j+1][i] * zScale;
				T.setCoordinate(coordNr, coord);	

				texCoord[0] = (y+(j+1)*multScaleY)/((float)ySize);
				texCoord[1] = (x+i*multScaleX)/((float)xSize);//0.0f;
				T.setTextureCoordinate(0, coordNr++, texCoord);

				// (5)
				coord[0] = (i+1) * gridScaleX;
				coord[2] = (j+1) * gridScaleY;
				coord[1] = values[j+1][i+1] * zScale;								
				T.setCoordinate(coordNr, coord);	

				texCoord[0] = (y+(j+1)*multScaleY)/((float)ySize);
				texCoord[1] = (x+(i+1)*multScaleX)/((float)xSize);
				T.setTextureCoordinate(0, coordNr++, texCoord);

				// (6)
				coord[0] = (i+1) * gridScaleX;
				coord[2] = j * gridScaleY;
				coord[1] = values[j][i+1] * zScale;
				T.setCoordinate(coordNr, coord);

				texCoord[0] = (y+j*multScaleY)/((float)ySize);//0.0f; 
				texCoord[1] = (x+(i+1)*multScaleX)/((float)xSize);

				T.setTextureCoordinate(0, coordNr++, texCoord);

			}
		}																						
		System.out.print(".. calculating normals..");
		/*
		   1 -- 2
			|    | 	Vertex normals and
			|    |   order of calculation
			3 -- 4		
		*/
		int p1, p2, p3, p4, p5, p6;
		Vector3f n1, n2, n3, n4;
		coordNr = 0;

	   for (int j = 0; j < h; j++) 
	   	for (int i = 0; i < w; i++) {				
				// Normal 1				
				// (j-1, i-1, 1) 	& (j-1, i-1, 2) 	& (j-1, i, 1) &
				// (j, i-1, 2) 	& (j, i, 1)			& (j, i , 2)

				p1 = (((y+(int)((j-1)*multDepth)) * (xSize-1) + (x+(int)((i-1)*multDepth))) * 2);

				if (p1 < 0 || p1 >= (xSize-1) * (ySize-1) * 2)
					p1 = normalNr;
					
				p2 = (((y+(int)((j-1)*multDepth)) * (xSize-1) + (x+(int)((i-1)*multDepth))) * 2) + 1;
				if (p2 < 0 || p2 >= (xSize-1) * (ySize-1) * 2)
					p2 = normalNr;

				p3 = (((y+(int)((j-1)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2);
				if (p3 < 0 || p3 >= (xSize-1)  * (ySize-1) * 2)
					p3 = normalNr;
					
				p4 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i-1)*multDepth))) * 2) + 1;
				if (p4 < 0 || p4 >= (xSize-1)  * (ySize-1) * 2)
					p4 = normalNr;

				p5 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2);
				if (p5 < 0 || p5 >= (xSize-1)  * (ySize-1) * 2)
					p5 = normalNr;

				p6 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2) + 1;
				if (p6 < 0 || p6 >= (xSize-1)  * (ySize-1) * 2)
					p6 = normalNr;

				n1 = normalAvg(normals[p1], normals[p2], normals[p3],
									normals[p4], normals[p5], normals[p6]);
									
				// Normal 2
				// (j-1, i ,1)	& (j-1, i, 2)	& (j-1, i+1, 1) &
				// (j, i, 2)	& (j, i+1, 1) 	& (j, i+1, 2)
				
				p1 = (((y+(int)((j-1)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2);
				if (p1 < 0 || p1 >= (xSize-1) * (ySize-1) * 2)
					p1 = normalNr;
					
				p2 = (((y+(int)((j-1)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2) + 1;
				if (p2 < 0 || p2 >= (xSize-1) * (ySize-1) * 2)
					p2 = normalNr;

				p3 = (((y+(int)((j-1)*multDepth)) * (xSize-1) + (x+(int)((i+1)*multDepth))) * 2);
				if (p3 < 0 || p3 >= (xSize-1)* (ySize-1) * 2)
					p3 = normalNr;

				p4 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2) + 1;
				if (p4 < 0 || p4 >= (xSize-1) * (ySize-1) * 2)
					p4 = normalNr;

				p5 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i+1)*multDepth))) * 2);
				if (p5 < 0 || p5 >= (xSize-1) * (ySize-1) * 2)
					p5 = normalNr;

				p6 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i+1)*multDepth))) * 2) + 1;
				if (p6 < 0 || p6 >= (xSize-1) * (ySize-1) * 2)
					p6 = normalNr;

				n2 = normalAvg(normals[p1], normals[p2], normals[p3],
									normals[p4], normals[p5], normals[p6]);									

				// Normal 3
				// (j, i-1, 1)		& (j, i-1, 2)	& (j, i, 1) &
				// (j+1, i-1, 2)	& (j+1, i, 1)	& (j+1, i, 2)

				p1 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i-1)*multDepth))) * 2);
				if (p1 < 0 || p1 >= (xSize-1) * (ySize-1)  * 2)
					p1 = normalNr;
					
				p2 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i-1)*multDepth))) * 2) + 1;
				if (p2 < 0 || p2 >= (xSize-1) * (ySize-1)  * 2)
					p2 = normalNr;

				p3 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2);
				if (p3 < 0 || p3 >= (xSize-1) * (ySize-1)  * 2)
					p3 = normalNr;

				p4 = (((y+(int)((j+1)*multDepth)) * (xSize-1) + (x+(int)((i-1)*multDepth))) * 2) + 1;
				if (p4 < 0 || p4 >= (xSize-1) * (ySize-1)  * 2)
					p4 = normalNr;

				p5 = (((y+(int)((j+1)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2);
				if (p5 < 0 || p5 >= (xSize-1) * (ySize-1)  * 2)
					p5 = normalNr;

				p6 = (((y+(int)((j+1)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2) + 1;
				if (p6 < 0 || p6 >= (xSize-1) * (ySize-1)  * 2)
					p6 = normalNr;
					
				n3 = normalAvg(normals[p1], normals[p2], normals[p3],
									normals[p4], normals[p5], normals[p6]);									

				// Normal 4
				// (j, i, 1)	& (j, i, 2)			& (j, i+1, 1) &
				// (j+1, i, 2)	& (j+1, i+1, 1)	& (j+1, i+1, 2)
				p1 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2);
				if (p1 < 0 || p1 >= (xSize-1) * (ySize-1) * 2)
					p1 = normalNr;
					
				p2 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2) + 1;
				if (p2 < 0 || p2 >= (xSize-1) * (ySize-1) * 2)
					p2 = normalNr;

				p3 = (((y+(int)((j)*multDepth)) * (xSize-1) + (x+(int)((i+1)*multDepth))) * 2);
				if (p3 < 0 || p3 >= (xSize-1) * (ySize-1) * 2)
					p3 = normalNr;

				p4 = (((y+(int)((j+1)*multDepth)) * (xSize-1) + (x+(int)((i)*multDepth))) * 2) + 1;
				if (p4 < 0 || p4 >= (xSize-1) * (ySize-1) * 2)
					p4 = normalNr;

				p5 = (((y+(int)((j+1)*multDepth)) * (xSize-1) + (x+(int)((i+1)*multDepth))) * 2);
				if (p5 < 0 || p5 >= (xSize-1) * (ySize-1) * 2)
					p5 = normalNr;

				p6 = (((y+(int)((j+1)*multDepth)) * (xSize-1) + (x+(int)((i+1)*multDepth))) * 2) + 1;
				if (p6 < 0 || p6 >= (xSize-1) * (ySize-1) * 2)
					p6 = normalNr;

				n4 = normalAvg(normals[p1], normals[p2], normals[p3],
									normals[p4], normals[p5], normals[p6]);
									
				// Set normals
				//         
				//		1 -- 2				3 -- 2         6
				//		|    |		--\		|  /         / |    
				//		|	 |		--/		| /    	   /   |
				//		3 -- 4				1         4 -- 5
				
				
				T.setNormal(coordNr++, n3);
				T.setNormal(coordNr++, n2);
				T.setNormal(coordNr++, n1);
				
				T.setNormal(coordNr++, n3);
				T.setNormal(coordNr++, n4);
				T.setNormal(coordNr++, n2);
								
			}				
		System.out.println(" done.");
		
		return T;
	} 			 
	
	/**
	 *	Mthode permettant de changer de mode
	 *  Textur ou Fil de Fer
	 **/
	public void switchMode(){
		if (wireFrame){
			wireFrame=false;
			for (Iterator it=arrayShape.iterator(); it.hasNext() ; ){
				Shape3D shtemp = (Shape3D)it.next();
				shtemp.setAppearance(createMaterialAppearance());
			}
		}
		else {
			wireFrame=true;
			for (Iterator it=arrayShape.iterator(); it.hasNext() ; ){
				Shape3D shtemp = (Shape3D)it.next();
				shtemp.setAppearance(wireFrameAppearance);
			}

		}

	}
	
	/**
	 * Mthode permettant de changer la qualit d'affichage du MNT
	 * en appliquant un facteur multiplicatif au distance de tous les DistanceLOD
	 * @param facteur multiplicatif (plus il est grand, meilleure est la qualit)
	 **/
	public void changeQuality(float newLODDistance){
		LODDistance=newLODDistance;
		
		for(Iterator it=arrayDL.iterator() ; it.hasNext() ;){
			DistanceLOD dl = (DistanceLOD)it.next();
			
			for (int i = 1; i <LODDepth; i++){
				dl.setDistance(i-1,i*LODDistance);
			}

		}
	}
	
	/** 
	 * Accessseur de xSize
	 * @return xSize
	 */
	public int getXSize(){
		return xSize;
	}
	
	/** 
	 * Accessseur de ySize
	 * @return ySize
	 */
	public int getYSize(){
		return ySize;
	}
	
	/** 
	 * Accessseur de la hauteur du Terrain 3D
	 * @param x coordonnes en x
	 * @param y coordonnes en y
	 * @return la hauteur au point [x,y]
	 */
	public float getHeight(int x,int y){
		return heightValues[y][x];
	}
}
